@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2014-2017 Laure Piqueras, Simon Conseil
+ * Copyright (C) 2014-2018 Laure Piqueras, Simon Conseil, Johan Richard
  *                         (CRAL - Observatoire de Lyon)
  *           (C) 2017-2018 European Southern Observatory
  *
@@ -104,13 +104,13 @@ muse_autocalib_mean_mad(float *data, int n, double x[3], int *indx)
   }
   mean = mean / n;
 
-  median = muse_autocalib_median(data,n,indx);
+  median = muse_autocalib_median(data, n, indx);
   for (i = 0; i < n; i++) {
     ind[i] = i;
     work[i] = fabs(data[indx[i]] - median);
   }
   x[0] = mean;
-  x[1] = muse_autocalib_median(work,n,ind) * 1.4826;
+  x[1] = muse_autocalib_median(work, n, ind) * 1.4826;
   cpl_free(ind);
   cpl_free(work);
 }
@@ -127,7 +127,7 @@ muse_autocalib_mean_sigma_clip(float *data, int n, double x[3], int nmax,
   muse_autocalib_mean(data, n, x, indx);
   x[2] = n;
   double med;
-  med =  muse_autocalib_median(data,n, indx);
+  med =  muse_autocalib_median(data, n, indx);
   clip_lo = med - (nclip_low * x[1]);
   clip_up = med + (nclip_up * x[1]);
 
@@ -165,7 +165,7 @@ muse_autocalib_mean_madsigma_clip(float *data, int n, double x[3], int nmax,
   muse_autocalib_mean_mad(data, n, x, indx);
   x[2] = n;
   double med;
-  med = muse_autocalib_median(data,n, indx);
+  med = muse_autocalib_median(data, n, indx);
   clip_lo = med - (nclip_low * x[1]);
   clip_up = med + (nclip_up * x[1]);
 
@@ -192,6 +192,7 @@ muse_autocalib_mean_madsigma_clip(float *data, int n, double x[3], int nmax,
   }
 }
 
+/* Compute the min and max of data */
 static void
 muse_autocalib_minmax(float data[], int n, int *indx, float res[])
 {
@@ -223,6 +224,7 @@ muse_autocalib_minmax(float data[], int n, int *indx, float res[])
   res[1] = max;
 }
 
+/* Compute the min and max of int data */
 static void
 muse_autocalib_minmax_int(unsigned short data[], int n, int *indx,
                           unsigned int res[])
@@ -255,10 +257,18 @@ muse_autocalib_minmax_int(unsigned short data[], int n, int *indx,
   res[1] = max;
 }
 
+/* Compute the mean flux for a slice.                                 *
+ *                                                                    *
+ * First, compute the mean flux for each spaxel that has enough       *
+ * values (50) along the wavelength axis.                             *
+ * Then, compute the mad-clipped mean/std of these fluxes, if we have *
+ * at least meanpix spaxels with a valid flux.                        */
 static void
 muse_autocalib_slice_mean(float *data, unsigned short *xpix, int n, double x[3],
                           int *indx, unsigned int min_pix)
 {
+  /* Find the min/max xpix values as we don't need *
+   * to iterate on spaxels without values.         */
   unsigned int minmax[2];
   muse_autocalib_minmax_int(xpix, n, indx, minmax);
 
@@ -275,6 +285,7 @@ muse_autocalib_slice_mean(float *data, unsigned short *xpix, int n, double x[3],
         ind[count++] = indx[j];
       }
     }
+    /* if we have at least 50 values for this spaxel, compute its mean flux */
     if (count > 50) {
       muse_autocalib_mean(data, count, x, ind);
       meanarr[meancount] = x[0];
@@ -297,6 +308,24 @@ muse_autocalib_slice_mean(float *data, unsigned short *xpix, int n, double x[3],
   cpl_free(meanarr);
 }
 
+/* apply the correction factor to the data buffers of a pixel table */
+static void
+muse_autocalib_applycorr(float *data, float *stat, int npix, unsigned char *ifu,
+                         unsigned char *sli, unsigned int *quad, float *corr,
+                         const char *idstring)
+{
+  cpl_msg_debug(idstring, "Applying corrections...");
+  cpl_size n;
+  #pragma omp parallel for default(none)                                       \
+          shared(corr, data, ifu, npix, quad, sli, stat, idstring)
+  for (n = 0; n < npix; n++) {
+    cpl_size k = MAPIDX(ifu[n], sli[n], quad[n]);
+    float c = corr[k];
+    data[n] *= c;
+    stat[n] *= c * c;
+  }
+}
+
 /*---------------------------------------------------------------------------*/
 /**
   @brief   Correct the background level of the slices (port from MPDAF).
@@ -317,8 +346,9 @@ muse_autocalib_slice_mean(float *data, unsigned short *xpix, int n, double x[3],
   @remark  This function adds a FITS header (@ref MUSE_HDR_PT_AUTOCAL_NAME) with
            the string "slice-median" to the pixel table, so that other functions
            can detect, if this type of autocalibration has already been carried
-           out. It will return without processing or error, if this is detected
-           in the input pixel table.
+           out. It will return without processing or error, if this keyword
+           already exists in the input pixel table, and the string contains a
+           known method.
            The comment of that header starts with the value of aClip.
 
   @error{set and return CPL_ERROR_NULL_INPUT, aPixtable is NULL}
@@ -329,12 +359,16 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
                             muse_table **aFactors)
 {
   cpl_ensure_code(aPixtable, CPL_ERROR_NULL_INPUT);
-  if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_AUTOCAL_NAME) &&
-      !strncmp(cpl_propertylist_get_string(aPixtable->header,
-                                           MUSE_HDR_PT_AUTOCAL_NAME),
-               "slice-median", 13)) {
-    cpl_msg_info(__func__, "pixel table already auto-calibrated: skipping "
-                 "correction");
+  /* check, if an autocalibration was already applied to this pixel table */
+  const char *kwstring = NULL;
+  if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_AUTOCAL_NAME)) {
+    kwstring = cpl_propertylist_get_string(aPixtable->header,
+                                           MUSE_HDR_PT_AUTOCAL_NAME);
+  }
+  if (kwstring && (!strncmp(kwstring, "slice-median", 13) ||
+                   !strncmp(kwstring, "user", 5))) {
+    cpl_msg_info(__func__, "pixel table already auto-calibrated (method %s): "
+                 "skipping correction", kwstring);
     return CPL_ERROR_NONE;
   }
 
@@ -376,6 +410,9 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
   unsigned short *xpix;
   muse_pixtable_origin_decode_all(aPixtable, &xpix, NULL, &ifu, &sli);
   cpl_size npix = muse_pixtable_get_nrow(aPixtable);
+
+  /* quad is the wavelength bin index. This is a poor name inherited *
+   * from a previous version of the autocalib code...                */
   unsigned int *quad = (unsigned int *)cpl_malloc(npix * sizeof(unsigned int));
 
   /* For each slice and wavelength bin, store the number of *
@@ -439,7 +476,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
   for (q = 0; q < nlbin; q++) {
     if (debug) {
       cpl_msg_debug(__func__, "Computing reference levels (lambda range: "
-                    "%.1f - %.1f)...", lbdabins[q], lbdabins[q+1]);
+                    "%.1f - %.1f Angstrom)...", lbdabins[q], lbdabins[q+1]);
     }
     int i, tot_count = 0;
     for (i = 0; i < kMuseNumIFUs; i++) {
@@ -453,6 +490,9 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
         k = MAPIDX(i+1, s+1, q+1);
         int slidx = kMuseSlicesPerCCD * i + s;
         if (npts[k] > 100) {
+          /* Compute the mean flux of the ifu/slice/wavelength bin,  *
+           * if we have at least 100 values, and if we have at least *
+           * min_pix_per_slice spaxels with a flux.                  */
           muse_autocalib_slice_mean(data, xpix, npts[k], x, indmap[k],
                                     min_pix_per_slice);
           if (x[2] > min_pix_per_slice) {
@@ -469,47 +509,72 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
         } else {
           slice_flux[slidx] = NAN;
         }
-        /* cpl_msg_debug(__func__, "  - SLICE %02hu : %f (%d)", s+1, */
-        /*     slice_flux[slidx], npts[k]); */
+        if (debug) {
+          cpl_msg_debug(__func__, "  - SLICE %02hu : %f (%d)",
+                        s+1, slice_flux[slidx], npts[k]);
+        }
       } /* for s */
 
+      /* Compute the mean flux for each half-IFU, as the mean of the *
+       * slice fluxes. */
       if (!slice_count1) {
         ifu_flux[2 * i] = 0.0;
       } else {
-        muse_autocalib_minmax(slice_flux, slice_count1, slice_ind1, minmax);
         muse_autocalib_mean_madsigma_clip(slice_flux, slice_count1, x, nmax,
                                           nclip_low, nclip_up, nstop,
                                           slice_ind1);
+        ifu_flux[2 * i] = x[0];
+
         if (debug) {
+          muse_autocalib_minmax(slice_flux, slice_count1, slice_ind1, minmax);
           cpl_msg_debug(__func__, "  - 1: Min max = %f %f / Mean = %f (%f, %d)",
                         minmax[0], minmax[1], x[0], x[1], (int)x[2]);
         }
-        ifu_flux[2 * i] = x[0];
-
-        if (debug && isnan(x[0])) {
-          cpl_msg_error(__func__, "Mean IFU flux is NAN");
+        if (isnan(x[0])) {
+          cpl_msg_warning(__func__, "Mean IFU flux is NAN (IFU %02d, lambda range"
+                          " %.1f - %.1f Angstrom)", i+1, lbdabins[q],
+                          lbdabins[q+1]);
         }
       }
 
+      /* Same for the second half-IFU. */
       if (!slice_count2) {
         ifu_flux[2 * i + 1] = 0.0;
       } else {
-        muse_autocalib_minmax(slice_flux, slice_count2, slice_ind2, minmax);
         muse_autocalib_mean_madsigma_clip(slice_flux, slice_count2, x, nmax,
                                           nclip_low, nclip_up, nstop,
                                           slice_ind2);
+        ifu_flux[2 * i + 1] = x[0];
+
         if (debug) {
+          muse_autocalib_minmax(slice_flux, slice_count2, slice_ind2, minmax);
           cpl_msg_debug(__func__, "  - 2: Min max = %f %f / Mean = %f (%f, %d)",
                         minmax[0], minmax[1], x[0], x[1], (int)x[2]);
         }
-        ifu_flux[2 * i + 1] = x[0];
-
-        if (debug && isnan(x[0])) {
-          cpl_msg_error(__func__, "Mean IFU flux is NAN");
+        if (isnan(x[0])) {
+          cpl_msg_warning(__func__, "Mean IFU flux is NAN (IFU %02d, lambda range"
+                          " %.1f - %.1f Angstrom)", i+1, lbdabins[q],
+                          lbdabins[q+1]);
         }
       }
 
-      /* Use mean ifu flux for slices without useful values */
+      /* if low number of valid slices test if the other half-IFU has more *
+       * valid slice fluxes and use them instead                           */
+      if ((slice_count1 < kMuseSlicesPerCCD/3) &&
+          (slice_count2 > kMuseSlicesPerCCD/3)) {
+        cpl_msg_debug(__func__, "Use right-half flux value of IFU %d for "
+                      "left-half", i+1);
+        ifu_flux[2 * i]=ifu_flux[2 * i + 1];
+      }
+
+      if ((slice_count1 > kMuseSlicesPerCCD/3) &&
+          (slice_count2 < kMuseSlicesPerCCD/3)) {
+        cpl_msg_debug(__func__, "Use left-half flux value of IFU %d for "
+                      "right-half", i+1);
+        ifu_flux[2 * i + 1]=ifu_flux[2 * i];
+      }
+
+      /* Use the mean ifu flux for slices with invalid value. */
       for (s = 0; s < kMuseSlicesPerCCD; s++) {
         int slidx = kMuseSlicesPerCCD * i + s;
         if (isnan(slice_flux[slidx])) {
@@ -522,14 +587,17 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
       } /* for s */
     } /* for i */
     if (!tot_count) {
-      cpl_msg_warning(__func__, "No values in this lambda range");
+      cpl_msg_warning(__func__, "No values in this lambda range (%.1f - %.1f "
+                      "Angstrom)", lbdabins[q], lbdabins[q+1]);
       continue;
     }
-    muse_autocalib_minmax(slice_flux, tot_count, tot_ind, minmax);
     if (debug) {
+      muse_autocalib_minmax(slice_flux, tot_count, tot_ind, minmax);
       cpl_msg_debug(__func__, "- Min max : %f %f", minmax[0], minmax[1]);
     }
 
+    /* Compute the total flux in the bin, as the sigma-clipped mean *
+     * of the slice fluxes.                                         */
     muse_autocalib_mean_sigma_clip(slice_flux, tot_count, x, nmax, nclip_low,
                                    nclip_up, nstop, tot_ind);
     if (debug) {
@@ -538,6 +606,10 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
     }
     tot_flux = x[0];
 
+    /* Compute the correction factors.                                     *
+     * First, compute the ratio between the total flux and the slice flux. *
+     * Then, apply a mad-sigma clipping to these corrections, to remove    *
+     * strongest ones.                                                     */
     if (debug) {
       cpl_msg_debug(__func__, "Computing corrections...");
     }
@@ -557,39 +629,47 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
           } else {
             slice_ind2[slice_count2++] = k;
           }
+        }
+        if (slice_flux[slidx] > 0) {
           corr[k] = tot_flux / slice_flux[slidx];
-          /* cpl_msg_debug(__func__, "  - SLICE %02hu : %f", s+1, corr[k]); */
+        }
+        if (debug) {
+          cpl_msg_debug(__func__, "  - SLICE %02hu : %f", s+1, corr[k]);
         }
       } /* for s */
       if ((slice_count1 + slice_count2) == 0) {
-        cpl_msg_warning(__func__, "No values in this IFU");
+        cpl_msg_warning(__func__, "No values in this IFU (%02d, lambda range: "
+                        "%.1f - %.1f Angstrom)", i+1, lbdabins[q],
+                        lbdabins[q+1]);
         continue;
       }
 
       if (slice_count1) {
-        muse_autocalib_minmax(corr, slice_count1, slice_ind1, minmax);
         muse_autocalib_mean_madsigma_clip(corr, slice_count1, x1, nmax,
                                           nclip_low, nclip_up, nstop,
                                           slice_ind1);
         if (debug) {
+          muse_autocalib_minmax(corr, slice_count1, slice_ind1, minmax);
           cpl_msg_debug(__func__, "  - 1: Min max = %f %f / Mean = %f (%f, %d)",
                         minmax[0], minmax[1], x1[0], x1[1], (int)x1[2]);
         }
       }
 
       if (slice_count2) {
-        muse_autocalib_minmax(corr, slice_count2, slice_ind2, minmax);
         muse_autocalib_mean_madsigma_clip(corr, slice_count2, x2, nmax,
                                           nclip_low, nclip_up, nstop,
                                           slice_ind2);
         if (debug) {
+          muse_autocalib_minmax(corr, slice_count2, slice_ind2, minmax);
           cpl_msg_debug(__func__, "  - 2: Min max = %f %f / Mean = %f (%f, %d)",
                         minmax[0], minmax[1], x2[0], x2[1], (int)x2[2]);
         }
       }
 
-      /* cpl_msg_debug(__func__, "  - Checking slice corrections (%.1f sigma clip)...", */
-      /*    aClip); */
+      if (debug) {
+        cpl_msg_debug(__func__, "  - Checking slice corrections "
+                      "(%.1f sigma clip)...", aClip);
+      }
       for (s = 0; s < kMuseSlicesPerCCD; s++) {
         k = MAPIDX(i+1, s+1, q+1);
         if (s < kMuseSlicesPerCCD/2) {
@@ -630,25 +710,24 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
   } /* if aFactors */
 
   cpl_msg_debug(__func__, "Cleaning and recording corrections...");
+  int nnan = 0;
+  for (k = 0; k < kMuseNumIFUs * kMuseSlicesPerCCD * nlbin; k++) {
+    if (isnan(corr[k])) {
+      corr[k] = 1.;
+      nnan++;
+    }
+  }
+  if (nnan > 0) {
+    cpl_msg_warning(__func__, "%d of %d wavelength bins in %d slices stay "
+                    "uncorrected!", nnan, kMuseNumIFUs * kMuseSlicesPerCCD * nlbin,
+                    kMuseNumIFUs * kMuseSlicesPerCCD);
+  }
   int i;
   for (i = 1; i <= kMuseNumIFUs; i++) {
     unsigned short s;
     for (s = 1; s <= kMuseSlicesPerCCD; s++) {
       for (q = 1; q <= nlbin; q++) {
         k = MAPIDX(i, s, q);
-        cpl_size kprev, knext;
-        if (q == 1) {
-          kprev = MAPIDX(i, s, q+1);
-          knext = MAPIDX(i, s, q+2);
-        } else if (q == nlbin) {
-          kprev = MAPIDX(i, s, q-1);
-          knext = MAPIDX(i, s, q-2);
-        } else {
-          kprev = MAPIDX(i, s, q-1);
-          knext = MAPIDX(i, s, q+1);
-        }
-        meanq = (corr[kprev] + corr[knext]) / 2.;
-        threshq = CPL_MAX(0.03, 3 * fabsf(corr[kprev] - corr[knext]));
         if (aFactors) {
           /* record slice properties in the table */
           cpl_table_set_int((*aFactors)->table, "ifu", k, i);
@@ -670,6 +749,20 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
             cpl_free(kw2);
           } /* if first slice */
         }
+
+        cpl_size kprev, knext;
+        if (q == 1) {
+          kprev = MAPIDX(i, s, q+1);
+          knext = MAPIDX(i, s, q+2);
+        } else if (q == nlbin) {
+          kprev = MAPIDX(i, s, q-1);
+          knext = MAPIDX(i, s, q-2);
+        } else {
+          kprev = MAPIDX(i, s, q-1);
+          knext = MAPIDX(i, s, q+1);
+        }
+        meanq = (corr[kprev] + corr[knext]) / 2.;
+        threshq = CPL_MAX(0.03, 3 * fabsf(corr[kprev] - corr[knext]));
         if (fabsf(corr[k] - meanq) > threshq) {
           if (aFactors) { /* store both original and revised factor */
             cpl_table_set_double((*aFactors)->table, "corr_orig", k, corr[k]);
@@ -685,15 +778,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
     } /* for s */
   } /* for i */
 
-  cpl_msg_debug(__func__, "Applying corrections...");
-  #pragma omp parallel for default(none) private(k)                            \
-          shared(corr, data, ifu, npix, quad, sli, stat)
-  for (n = 0; n < npix; n++) {
-    k = MAPIDX(ifu[n], sli[n], quad[n]);
-    float c = corr[k];
-    data[n] *= c;
-    stat[n] *= c * c;
-  }
+  muse_autocalib_applycorr(data, stat, npix, ifu, sli, quad, corr, __func__);
   muse_utils_memory_dump("ac_max");
 
   /* cleanup the remaining allocations */
@@ -727,6 +812,139 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
   return CPL_ERROR_NONE;
 } /* muse_autocalib_slice_median() */
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief   Apply user self-calibration corrections to a pixel table.
+  @param   aPixtable   input pixel table
+  @param   aFactors    input table containing the correction factors
+  @return  CPL_ERROR_NONE on success or a cpl_error_code on failure
+
+  This uses the user-provided multiplicative corrections to apply to each slice.
+
+  The format of aFactors is expected to be the same as the one produced by
+  @ref muse_autocalib_slice_median().
+
+  @remark  This function adds a FITS header (@ref MUSE_HDR_PT_AUTOCAL_NAME) with
+           the string "user" to the pixel table, so that other functions
+           can detect, if this manual autocalibration has already been carried
+           out. It will return without processing or error, if this keyword
+           already exists in the input pixel table, and the string contains a
+           known method.
+
+  @error{set and return CPL_ERROR_NULL_INPUT,
+         aPixtable or aFactors or one of their components are NULL}
+  @error{set and return CPL_ERROR_DATA_NOT_FOUND,
+         aFactors does not contain a column "corr" in its table component}
+ */
+/*---------------------------------------------------------------------------*/
+cpl_error_code
+muse_autocalib_apply(muse_pixtable *aPixtable, muse_table *aFactors)
+{
+  cpl_ensure_code(aPixtable && aPixtable->table &&
+                  aFactors && aFactors->header && aFactors->table,
+                  CPL_ERROR_NULL_INPUT);
+  cpl_ensure_code(cpl_table_has_column(aFactors->table, "corr"),
+                  CPL_ERROR_DATA_NOT_FOUND);
+  /* check, if an autocalibration was already applied to this pixel table */
+  const char *kwstring = NULL;
+  if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_AUTOCAL_NAME)) {
+    kwstring = cpl_propertylist_get_string(aPixtable->header,
+                                           MUSE_HDR_PT_AUTOCAL_NAME);
+  }
+  if (kwstring && (!strncmp(kwstring, "slice-median", 13) ||
+                   !strncmp(kwstring, "user", 5))) {
+    cpl_msg_info(__func__, "pixel table already auto-calibrated (method %s): "
+                 "skipping correction", kwstring);
+    return CPL_ERROR_NONE;
+  }
+
+  /* read lambda_bins from aFactors header */
+  int nlbin = 1;
+  /* first search for number of lambda bins */
+  while (nlbin < 1000) { /* arbitrary limit, should be 19 or 20... */
+    char *kw1 = cpl_sprintf("ESO DRS MUSE LAMBDA%d MIN", nlbin);
+    if (!cpl_propertylist_has(aFactors->header, kw1)) {
+      cpl_free(kw1);
+      break;
+    }
+    nlbin++;
+    cpl_free(kw1);
+  }
+  nlbin--;
+  if (!nlbin) {
+    char *str = cpl_sprintf("No keywords \"ESO DRS MUSE LAMBDAi MIN/MAX\" found"
+                            " in %s header!", MUSE_TAG_ACAL_FACTORS);
+    cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_INPUT, "%s", str);
+    cpl_msg_error(__func__, "%s", str);
+    cpl_free(str);
+    return CPL_ERROR_ILLEGAL_INPUT;
+  }
+  cpl_boolean isAO = muse_pfits_get_mode(aPixtable->header)
+                   >= MUSE_MODE_WFM_AO_E;
+  cpl_msg_info(__func__, "Running self-calibration, using %d lambda ranges "
+               "(%s data) and user table", nlbin, isAO ? "AO" : "non-AO");
+  int nrow = cpl_table_get_nrow(aFactors->table),
+      nexpected = kMuseSlicesPerCCD * kMuseNumIFUs * nlbin;
+  if (nrow != nexpected) {
+    cpl_msg_warning(__func__, "%s contains %d instead of %d rows!",
+                    MUSE_TAG_ACAL_FACTORS, nrow, nexpected);
+  }
+
+  /* now set up the buffer and fill it with the wavelength ranges */
+  float *lbdabins = (float *)cpl_malloc((nlbin+1) * sizeof(float));
+  unsigned short q = 0;
+  for (q = 1; q < nlbin+1; q++) {
+    char *kw1 = cpl_sprintf("ESO DRS MUSE LAMBDA%d MIN", q),
+         *kw2 = cpl_sprintf("ESO DRS MUSE LAMBDA%d MAX", q);
+    lbdabins[q-1] = cpl_propertylist_get_double(aFactors->header, kw1);
+    lbdabins[q] = cpl_propertylist_get_double(aFactors->header, kw2);
+    cpl_free(kw1);
+    cpl_free(kw2);
+  }
+
+  unsigned char *ifu, *sli;
+  unsigned short *xpix;
+  muse_pixtable_origin_decode_all(aPixtable, &xpix, NULL, &ifu, &sli);
+
+  float *data = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_DATA),
+        *stat = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_STAT),
+        *lbda = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_LAMBDA);
+
+  cpl_size n, npix = muse_pixtable_get_nrow(aPixtable);
+  unsigned int *quad = (unsigned int *)cpl_malloc(npix * sizeof(unsigned int));
+
+  /* loop through all pixels and fill the remaining buffers */
+  cpl_msg_debug(__func__, "Computing lambda indices...");
+  #pragma omp parallel for default(none) private(q)                            \
+          shared(lbda, lbdabins, nlbin, npix, quad)
+  for (n = 0; n < npix; n++) {
+    for (q = 1; q < nlbin+1; q++) {
+      if ((lbda[n] >= lbdabins[q-1]) && (lbda[n] < lbdabins[q])) {
+        quad[n] = q;
+        break;
+      }
+    }
+  }
+  cpl_free(lbdabins);
+
+  cpl_table_cast_column(aFactors->table, "corr", "corr_float", CPL_TYPE_FLOAT);
+  float *corr = cpl_table_get_data_float(aFactors->table, "corr_float");
+  muse_autocalib_applycorr(data, stat, npix, ifu, sli, quad, corr, __func__);
+  cpl_table_erase_column(aFactors->table, "corr_float");
+  cpl_free(xpix);
+  cpl_free(ifu);
+  cpl_free(sli);
+  cpl_free(quad);
+
+  /* add the header */
+  cpl_propertylist_update_string(aPixtable->header, MUSE_HDR_PT_AUTOCAL_NAME,
+                                 "user");
+  cpl_propertylist_set_comment(aPixtable->header, MUSE_HDR_PT_AUTOCAL_NAME,
+                               "used user table for slice autocalibration");
+
+  return CPL_ERROR_NONE;
+} /* muse_autocalib_apply() */
+
 /*---------------------------------------------------------------------------*/
 /**
   @brief   Create Select spaxels to be considered as sky for autocalibration.
