@@ -27,6 +27,7 @@
  *                             Includes                                       *
  *----------------------------------------------------------------------------*/
 #ifdef HAVE_READLINK
+#define _DEFAULT_SOURCE
 #define _BSD_SOURCE /* get readlink() from unistd.h */
 #include <unistd.h> /* readlink(), has to be included before cpl.h */
 #endif
@@ -340,24 +341,34 @@ muse_cplimage_slope_window(const cpl_image *aImage, const cpl_size *aWindow)
    of total valid pixels.
 
    @cpl_ensure{aImage != NULL, CPL_ERROR_NULL_INPUT, 0.0}
-
 */
 /*----------------------------------------------------------------------------*/
 double
-muse_cplimage_get_percentile(const cpl_image *aImage, double aFraction) {
+muse_cplimage_get_percentile(const cpl_image *aImage, double aFraction)
+{
   cpl_ensure(aImage != NULL, CPL_ERROR_NULL_INPUT, 0.0);
 
   cpl_array *a = muse_cplarray_new_from_image(aImage);
   muse_cplarray_erase_invalid(a);
   cpl_size n = cpl_array_get_size(a);
   muse_cplarray_sort(a, TRUE);
-  if (aFraction < 0) aFraction = 0;
-  if (aFraction > 1) aFraction = 1;
-  n = lround(n * aFraction);
-  double res = cpl_array_get(a, n-1, NULL);
+  if (aFraction < 0) {
+    aFraction = 0.;
+  }
+  if (aFraction > 1) {
+    aFraction = 1.;
+  }
+  /* array index of where the percentile is located */
+  cpl_size i = lround(n * aFraction) - 1;
+  if (i < 0) {
+    i = 0;
+  }
+  if (i >= n) {
+    i = n - 1;
+  }
+  double res = cpl_array_get(a, i, NULL);
   cpl_array_delete(a);
   return res;
-
 }
 
 /*----------------------------------------------------------------------------*/
