@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2015 European Southern Observatory
+ * Copyright (C) 2005-2018 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,10 +55,12 @@
 
 /**@{*/
 
+/*----------------------------------------------------------------------------*/
 /**
-   @brief Check the pixels for one arc line
-   @param aPixtable The pixel table with pixels around the line
-   @param aLambda   The arc line reference wavelength
+   @private
+   @brief Check the pixels for one arc line.
+   @param aPixtable   the pixel table with pixels around the line
+   @param aLambda     the arc line reference wavelength
 
    The basic check is done with a gaussian fit, which also gives the
    total flux for this line. Lines with a low flux, and lines with an
@@ -71,14 +73,18 @@
    the constant values of the aLambda parameter, the calculated total
    line flux, and the calculated continuum background.
  */
+/*----------------------------------------------------------------------------*/
 static cpl_size
-muse_lsf_check_arc_line(cpl_table *aPixtable, double aLambda) {
+muse_lsf_check_arc_line(cpl_table *aPixtable, double aLambda)
+{
   cpl_size nrows = cpl_table_get_nrow(aPixtable);
   /* If we have too few data pointe, remove the line */
   if (nrows <= 100) {
     return 0;
   }
   cpl_errorstate prestate = cpl_errorstate_get();
+  /* get origin before any more (or all?!) rows are removed */
+  uint32_t origin = cpl_table_get_int(aPixtable, MUSE_PIXTABLE_ORIGIN, 0, NULL);
 
   /* wrap table columns into vectors for the Gaussian fit */
   cpl_vector *pos = cpl_vector_new(nrows);
@@ -125,11 +131,14 @@ muse_lsf_check_arc_line(cpl_table *aPixtable, double aLambda) {
       cpl_table_select_row(aPixtable, i);
     }
   }
-  cpl_table_erase_selected(aPixtable);
-  nrows = cpl_table_get_nrow(aPixtable);
   cpl_vector_delete(pos);
   cpl_vector_delete(val);
   cpl_vector_delete(err);
+  cpl_table_erase_selected(aPixtable);
+  nrows = cpl_table_get_nrow(aPixtable);
+  if (nrows == 0) {
+    return 0;
+  }
 
   cpl_table_fill_column_window_float(aPixtable, "line_lambda",
                                      0, nrows, xc);
@@ -150,9 +159,6 @@ muse_lsf_check_arc_line(cpl_table *aPixtable, double aLambda) {
   /* If the estimated line width is too far away from our expectation,
      remove the line */
   if ((sigma > 3) || (sigma < 0.7)) {
-    uint32_t origin = (uint32_t)cpl_table_get_int(aPixtable,
-                                                  MUSE_PIXTABLE_ORIGIN,
-                                                  0, NULL);
     int i_ifu = muse_pixtable_origin_get_ifu(origin);
     int i_slice = muse_pixtable_origin_get_slice(origin);
     cpl_msg_debug(__func__, "Slice %2i.%02i: "
@@ -161,9 +167,11 @@ muse_lsf_check_arc_line(cpl_table *aPixtable, double aLambda) {
     return 0;
   }
   return nrows;
-}
+} /* muse_lsf_check_arc_line() */
 
+/*----------------------------------------------------------------------------*/
 /**
+   @private
    @brief Select the pixels for one arc line
    @param aPixtable The pixel table for one slice
    @param aLambda   The arc line reference wavelength
@@ -180,42 +188,50 @@ muse_lsf_check_arc_line(cpl_table *aPixtable, double aLambda) {
    <tt>line_background</tt>
 
    If no line is found, NULL is returned.
-
  */
+/*----------------------------------------------------------------------------*/
 static cpl_table *
-select_arc_line(muse_pixtable *aPixtable, double aLambda, double aWindow) {
+muse_lsf_select_arc_line(muse_pixtable *aPixtable, double aLambda,
+                         double aWindow)
+{
   cpl_size low = muse_cpltable_find_sorted(aPixtable->table, MUSE_PIXTABLE_LAMBDA,
                                            aLambda - aWindow);
   cpl_size high = muse_cpltable_find_sorted(aPixtable->table, MUSE_PIXTABLE_LAMBDA,
                                             aLambda + aWindow);
+  if (high == low) {
+    return NULL;
+  }
   cpl_table *sel = cpl_table_extract(aPixtable->table, low + 1, high - low);
   cpl_table_select_all(sel);
   cpl_table_and_selected_int(sel, "dq", CPL_NOT_EQUAL_TO, 0);
   cpl_table_erase_selected(sel);
+  if (cpl_table_get_nrow(sel) == 0) {
+    cpl_table_delete(sel);
+    return NULL;
+  }
 
   cpl_table_new_column(sel, "line_lambda", CPL_TYPE_FLOAT);
   cpl_table_new_column(sel, "line_flux", CPL_TYPE_FLOAT);
   cpl_table_new_column(sel, "line_background", CPL_TYPE_FLOAT);
-  
-  cpl_size nrows = muse_lsf_check_arc_line(sel, aLambda);
 
+  cpl_size nrows = muse_lsf_check_arc_line(sel, aLambda);
   if (nrows == 0) {
     cpl_table_delete(sel);
     return NULL;
   }
   return sel;
-}
-
+} /* muse_lsf_select_arc_line() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Read images and combine the arc lines into one pixtable.
-   @param aImages   List of arc images
-   @param aTrace    Table containing the trace solution
-   @param aWave     Table containing the wavelength calibration solution
-   @param aArcLines ARC reference line table
-   @param aQuality  Minimal line quality to select
-   @param aWindow   Wavelength window around each line
-   @retval Combined pixel table or NULL on error
+   @param aImages     list of arc images
+   @param aTrace      table containing the trace solution
+   @param aWave       table containing the wavelength calibration solution
+   @param aArcLines   arc reference line table
+   @param aQuality    minimal line quality to select
+   @param aWindow     wavelength window around each line
+   @return combined pixel table or NULL on error
    @cpl_ensure{aImages, CPL_ERROR_NULL_INPUT, NULL}
    @cpl_ensure{aTrace, CPL_ERROR_NULL_INPUT, NULL}
    @cpl_ensure{aWave, CPL_ERROR_NULL_INPUT, NULL}
@@ -229,6 +245,7 @@ select_arc_line(muse_pixtable *aPixtable, double aLambda, double aWindow) {
    <tt>line_background</tt>, containing the reference wavelength, the
    estimated flux, and the background level of the corresponding line.
  */
+/*----------------------------------------------------------------------------*/
 muse_pixtable *
 muse_lsf_create_arcpixtable(muse_imagelist *aImages,
                             cpl_table *aTrace, cpl_table *aWave,
@@ -295,9 +312,9 @@ muse_lsf_create_arcpixtable(muse_imagelist *aImages,
                        aWindow, pt)
 #endif
         for (i_slice = 0; i_slice < n_slices; i_slice++) {
-          cpl_table *sel = select_arc_line(slice_pixtable[i_slice],
-                                           lambda, aWindow);
-          if (sel != NULL) {
+          cpl_table *sel = muse_lsf_select_arc_line(slice_pixtable[i_slice],
+                                                    lambda, aWindow);
+          if (sel) {
 #if 0 /* no parallelization here, not necessary any more */
             #pragma omp critical(construct_pixtable)
 #endif
@@ -321,17 +338,19 @@ muse_lsf_create_arcpixtable(muse_imagelist *aImages,
   return pt;
 } /* muse_lsf_create_arcpixtable() */
 
+/*----------------------------------------------------------------------------*/
 /**
+   @private
    @brief Do a polynomial fit on an arc pixtable
    @param aPixtable      special pixel table with arc lines
    @param aOrderLsf      polynomial order in x (LSF) direction
    @param aOrderLambda   polynomial order in y (arc line wavelength) direction
-   @return Fitted polynomial, or NULL or errors
-   @cpl_ensure{aPixtable["d_lambda"] != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL}
-   @cpl_ensure{aPixtable["line_lambda"] != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL}
-   @cpl_ensure{aPixtable["data"] != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL}
-   @cpl_ensure{aPixtable["line_flux"] != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL}
-   @cpl_ensure{aPixtable["line_background"] != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL}
+   @return fitted polynomial, or NULL or error
+   @cpl_ensure{aPixtable["d_lambda"], CPL_ERROR_ILLEGAL_INPUT, NULL}
+   @cpl_ensure{aPixtable["line_lambda"], CPL_ERROR_ILLEGAL_INPUT, NULL}
+   @cpl_ensure{aPixtable["data"], CPL_ERROR_ILLEGAL_INPUT, NULL}
+   @cpl_ensure{aPixtable["line_flux"], CPL_ERROR_ILLEGAL_INPUT, NULL}
+   @cpl_ensure{aPixtable["line_background"], CPL_ERROR_ILLEGAL_INPUT, NULL}
 
    The fit is done in two dimensions: x direction is the pixel wavelength,
    relative to the arc line (column <tt>d_lambda</tt>),
@@ -344,6 +363,7 @@ muse_lsf_create_arcpixtable(muse_imagelist *aImages,
    If there are not enough data points, or if the fit fails for other
    reasons, the funtion returns NULL, without setting an error.
  */
+/*----------------------------------------------------------------------------*/
 static cpl_polynomial *
 muse_lsf_fit_polynomial(cpl_table *aPixtable, int aOrderLsf, int aOrderLambda)
 {
@@ -353,15 +373,15 @@ muse_lsf_fit_polynomial(cpl_table *aPixtable, int aOrderLsf, int aOrderLambda)
     return NULL;
   }
   double *p_dlambda = cpl_table_get_data_double(aPixtable, "d_lambda");
-  cpl_ensure(p_dlambda != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL);
+  cpl_ensure(p_dlambda, CPL_ERROR_ILLEGAL_INPUT, NULL);
   float *p_line = cpl_table_get_data_float(aPixtable, "line_lambda");
-  cpl_ensure(p_line != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL);
+  cpl_ensure(p_line, CPL_ERROR_ILLEGAL_INPUT, NULL);
   float *p_data = cpl_table_get_data_float(aPixtable, MUSE_PIXTABLE_DATA);
-  cpl_ensure(p_data != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL);
+  cpl_ensure(p_data, CPL_ERROR_ILLEGAL_INPUT, NULL);
   float *p_flux = cpl_table_get_data_float(aPixtable, "line_flux");
-  cpl_ensure(p_flux != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL);
+  cpl_ensure(p_flux, CPL_ERROR_ILLEGAL_INPUT, NULL);
   float *p_background = cpl_table_get_data_float(aPixtable, "line_background");
-  cpl_ensure(p_background != NULL, CPL_ERROR_ILLEGAL_INPUT, NULL);
+  cpl_ensure(p_background, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
   cpl_matrix *coord = cpl_matrix_new(2, nrows);
   cpl_vector *data = cpl_vector_new(nrows);
@@ -387,15 +407,17 @@ muse_lsf_fit_polynomial(cpl_table *aPixtable, int aOrderLsf, int aOrderLambda)
   }
 
   return p;
-}
+} /* muse_lsf_fit_polynomial() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Compute the LSF for all wavelengths of one slice.
    @param aPixtable              Special pixel table with arc lines.
    @param aLsfImage              Image to store the LSF.
    @param aWCS                   The WCS of the image.
    @param aLsfRegressionWindow   The Regression window in LSF direction.
-   @retval CPL_ERROR_NONE Everything went OK
+   @return CPL_ERROR_NONE if everything went OK or another CPL error code on
+           failure
    @cpl_ensure_code{aPixtable, CPL_ERROR_NULL_INPUT}
    @cpl_ensure_code{aLsfImage, CPL_ERROR_NULL_INPUT}
    @cpl_ensure_code{aWCS, CPL_ERROR_NULL_INPUT}
@@ -416,6 +438,7 @@ muse_lsf_fit_polynomial(cpl_table *aPixtable, int aOrderLsf, int aOrderLambda)
    LSF. If a fit did not succeed, the pixels in the according image
    column are set invalid.
  */
+/*----------------------------------------------------------------------------*/
 cpl_error_code
 muse_lsf_fit_slice(const muse_pixtable *aPixtable, cpl_image *aLsfImage,
                    muse_wcs *aWCS, double aLsfRegressionWindow)
@@ -431,8 +454,7 @@ muse_lsf_fit_slice(const muse_pixtable *aPixtable, cpl_image *aLsfImage,
   }
   /* work on a locally duplicated table (since we mess with the columns) */
   cpl_table *pixtable = cpl_table_duplicate(aPixtable->table);
-  uint32_t origin = (uint32_t)cpl_table_get_int(pixtable,
-                                                MUSE_PIXTABLE_ORIGIN, 0, NULL);
+  uint32_t origin = cpl_table_get_int(pixtable, MUSE_PIXTABLE_ORIGIN, 0, NULL);
   int i_ifu = muse_pixtable_origin_get_ifu(origin);
   int i_slice = muse_pixtable_origin_get_slice(origin);
   cpl_msg_info(__func__, "processing slice %2i.%02i"
@@ -467,7 +489,7 @@ muse_lsf_fit_slice(const muse_pixtable *aPixtable, cpl_image *aLsfImage,
                                               x_ref + aLsfRegressionWindow);
     cpl_table *selected = cpl_table_extract(pixtable, low+1, high-low);
     cpl_polynomial *p = muse_lsf_fit_polynomial(selected, 2, 3);
-    if (p != NULL) {
+    if (p) {
       cpl_vector *c = cpl_vector_new(2);
       cpl_vector_set(c, 0, x);
       cpl_size i_lambda;
@@ -508,16 +530,18 @@ muse_lsf_fit_slice(const muse_pixtable *aPixtable, cpl_image *aLsfImage,
     }
   }
   return CPL_ERROR_NONE;
-}
+} /* muse_lsf_fit_slice() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Create a new LSF datacube.
-   @param aLsfHalfRange Half width of the range in LSF direction
-   @param aNLsf         Number of data points in LSF direction
-   @param aNLambda      Number of data points in line wavelength direction
-   @param aHeader       Header to derive the LSF cube header from (can be NULL)
+   @param aLsfHalfRange   Half width of the range in LSF direction
+   @param aNLsf           Number of data points in LSF direction
+   @param aNLambda        Number of data points in line wavelength direction
+   @param aHeader         Header to derive the LSF cube header from (can be NULL)
    @return The newly created data cube.
  */
+/*----------------------------------------------------------------------------*/
 muse_lsf_cube *
 muse_lsf_cube_new(double aLsfHalfRange, cpl_size aNLsf, cpl_size aNLambda,
                   const cpl_propertylist *aHeader)
@@ -550,10 +574,12 @@ muse_lsf_cube_new(double aLsfHalfRange, cpl_size aNLsf, cpl_size aNLambda,
   return lsf;
 } /* muse_lsf_cube_new() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Deallocate the memory for the LSF cube.
    @param aLsfCube The LSF cube to deallocate
-*/
+ */
+/*----------------------------------------------------------------------------*/
 void
 muse_lsf_cube_delete(muse_lsf_cube *aLsfCube)
 {
@@ -564,13 +590,15 @@ muse_lsf_cube_delete(muse_lsf_cube *aLsfCube)
   cpl_imagelist_delete(aLsfCube->img);
   cpl_free(aLsfCube->wcs);
   cpl_free(aLsfCube);
-}
+} /* muse_lsf_cube_delete() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Save the LSF cube to disk.
    @param aLsfCube  The LSF cube
    @param aFileName FITS file name
  */
+/*----------------------------------------------------------------------------*/
 cpl_error_code
 muse_lsf_cube_save(muse_lsf_cube *aLsfCube, const char *aFileName)
 {
@@ -658,27 +686,34 @@ muse_lsf_cube_load_all(muse_processing *aProcessing)
   return lsf;
 } /* muse_lsf_cube_load_all() */
 
-/** @brief Delete all LSF cubes.
-    @param aLsfCube Array of 48 data cubes
+/*----------------------------------------------------------------------------*/
+/**
+   @brief Delete all LSF cubes.
+   @param aLsfCube   array of 48 data cubes
  */
+/*----------------------------------------------------------------------------*/
 void
-muse_lsf_cube_delete_all(muse_lsf_cube **aLsfCube) {
-  if (aLsfCube != NULL) {
-    int ifu;
-    for (ifu = 0; ifu < kMuseNumIFUs; ifu++) {
-      muse_lsf_cube_delete(aLsfCube[ifu]);
-    }
-    cpl_free(aLsfCube);
+muse_lsf_cube_delete_all(muse_lsf_cube **aLsfCube)
+{
+  if (!aLsfCube) {
+   return;
   }
-}
+  int ifu;
+  for (ifu = 0; ifu < kMuseNumIFUs; ifu++) {
+    muse_lsf_cube_delete(aLsfCube[ifu]);
+  }
+  cpl_free(aLsfCube);
+} /* muse_lsf_cube_delete_all() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Load a LSF cube for one single IFU from disk.
-   @param aFileName FITS file name
-   @param aIFU      the IFU number
-   @retval The LSF cube.
+   @param aFileName   FITS file name
+   @param aIFU        the IFU number
+   @return the LSF cube.
    @cpl_ensure{aFileName, CPL_ERROR_NULL_INPUT, NULL}
  */
+/*----------------------------------------------------------------------------*/
 muse_lsf_cube *
 muse_lsf_cube_load(const char *aFileName, unsigned char aIFU)
 {
@@ -730,16 +765,18 @@ muse_lsf_cube_load(const char *aFileName, unsigned char aIFU)
   return lsf;
 } /* muse_lsf_cube_load() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Create an average image from all LSF cubes
-   @param aLsfCube Array of all LSF cubes
-   @param aPixtable Pixtable for the weighting, or NULL
+   @param aLsfCube    array of all LSF cubes
+   @param aPixtable   pixtable for the weighting, or NULL
    @return Average LSF image
 
    All images from all cubes are squeezed into one image. If a
    pixtable is given, each IFU/slice is weighted with the number of
    entries in the pixtable for this IFU/slice.
  */
+/*----------------------------------------------------------------------------*/
 cpl_image *
 muse_lsf_average_cube_all(muse_lsf_cube **aLsfCube, muse_pixtable *aPixtable)
 {
@@ -755,11 +792,11 @@ muse_lsf_average_cube_all(muse_lsf_cube **aLsfCube, muse_pixtable *aPixtable)
     }
   }
 
-  if (aPixtable != NULL) {
+  if (aPixtable) {
     cpl_size n_rows = muse_pixtable_get_nrow(aPixtable);
     cpl_size i_row;
-    uint32_t *origin = (uint32_t*)
-      cpl_table_get_data_int(aPixtable->table, MUSE_PIXTABLE_ORIGIN);
+    uint32_t *origin = (uint32_t *)cpl_table_get_data_int(aPixtable->table,
+                                                          MUSE_PIXTABLE_ORIGIN);
     for (i_row = 0; i_row < n_rows; i_row++) {
       int i_ifu = muse_pixtable_origin_get_ifu(origin[i_row]);
       int i_slice = muse_pixtable_origin_get_slice(origin[i_row]);
@@ -796,26 +833,41 @@ muse_lsf_average_cube_all(muse_lsf_cube **aLsfCube, muse_pixtable *aPixtable)
       }
     }
   }
-  if ((img != NULL) && (wsum > 0)) {
+  if ((img) && (wsum > 0)) {
     cpl_image_divide_scalar(img, wsum);
     return img;
   } else {
     cpl_image_delete(img);
     return NULL;
   }
-}
+} /* muse_lsf_average_cube_all() */
 
-// XXX this should be documented and actually check that all WCSs are identical!
+/*----------------------------------------------------------------------------*/
+/**
+   @brief Get the world coordinate representative of all LSF cubes.
+   @param aLsfCubes   the array of LSF cubes
+   @return a representative WCS or NULL on error
+
+   Since apparently nobody messed with the cube's WCS, it is enough to just
+   return the first WCS found.
+
+   XXX this should really check that all WCSs are identical!
+
+   @error{set CPL_ERROR_NULL_INPUT\, return NULL, aLsfCubes are NULL}
+ */
+/*----------------------------------------------------------------------------*/
 muse_wcs *
-muse_lsf_cube_get_wcs_all(muse_lsf_cube **aLsfCube) {
-  cpl_size ifu;
+muse_lsf_cube_get_wcs_all(muse_lsf_cube **aLsfCubes)
+{
+  cpl_ensure(aLsfCubes, CPL_ERROR_NULL_INPUT, NULL);
+  int ifu;
   for (ifu = 0; ifu < kMuseNumIFUs; ifu++) {
-    if (aLsfCube[ifu] != NULL) {
-      return aLsfCube[ifu]->wcs;
+    if (aLsfCubes[ifu]) {
+      return aLsfCubes[ifu]->wcs;
     }
   }
   return NULL;
-}
+} /* muse_lsf_cube_get_wcs_all() */
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -852,7 +904,6 @@ muse_lsf_fold_rectangle(cpl_image *aLsfImage, const muse_wcs *aWCS,
   return CPL_ERROR_NONE;
 } /* muse_lsf_fold_rectangle() */
 
-
 /*----------------------------------------------------------------------------*/
 /**
    @brief Apply the LSF to a number of data points of one slice
@@ -861,7 +912,8 @@ muse_lsf_fold_rectangle(cpl_image *aLsfImage, const muse_wcs *aWCS,
    @param aVal     Array with wavelengths [A]. Will be replaced by the
                    normalized LSF data points
    @param aLambda  Line wavelength reference [A].
-   @retval CPL_ERROR_NONE Everything went OK
+   @return CPL_ERROR_NONE if everything went OK or another CPL error code on
+           failure
    @cpl_ensure_code{aLsfImage, CPL_ERROR_NULL_INPUT}
    @cpl_ensure_code{aWCS, CPL_ERROR_NULL_INPUT}
    @cpl_ensure_code{aVal, CPL_ERROR_NULL_INPUT}
@@ -936,8 +988,7 @@ muse_lsf_apply(const cpl_image *aLsfImage, const muse_wcs *aWCS,
   cpl_array_delete(lsf_lambda);
   cpl_array_delete(lsf_values);
   return CPL_ERROR_NONE;
-}
-
+} /* muse_lsf_apply() */
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -957,8 +1008,7 @@ muse_lsf_apply(const cpl_image *aLsfImage, const muse_wcs *aWCS,
 /*----------------------------------------------------------------------------*/
 cpl_error_code
 muse_lsf_apply_slice(muse_pixtable *aPixtable, double aWavelength,
-                     double aFlux,
-                     cpl_image *aLsfImage, muse_wcs *aLsfWCS)
+                     double aFlux, cpl_image *aLsfImage, muse_wcs *aLsfWCS)
 {
   cpl_propertylist *order = cpl_propertylist_new();
   cpl_propertylist_append_bool(order, MUSE_PIXTABLE_LAMBDA, CPL_FALSE);
@@ -996,6 +1046,6 @@ muse_lsf_apply_slice(muse_pixtable *aPixtable, double aWavelength,
   }
 
   return CPL_ERROR_NONE;
-}
+} /* muse_lsf_apply_slice() */
 
 /**@}*/
