@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2016 European Southern Observatory
+ * Copyright (C) 2005-2018 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ typedef unsigned int uint32_t;
 #define MUSE_PIXTABLE_YPOS    "ypos" /**< y-position column of a MUSE pixel table */
 #define MUSE_PIXTABLE_LAMBDA  "lambda" /**< wavelength column of a MUSE pixel table */
 #define MUSE_PIXTABLE_ORIGIN  "origin" /**< origin column of a MUSE pixel table */
+
 /* relative weight of a pixel (used for combining exposures) */
 #define MUSE_PIXTABLE_WEIGHT  "weight" /**< weight column of a MUSE pixel table (optional) */
 
@@ -194,6 +195,9 @@ extern const muse_cpltable_def muse_pixtable_def[];
 /** empirical DAR correction done, relative to this reference wavelength [Angstrom] */
 #define MUSE_HDR_PT_DAR_CORR           "ESO DRS MUSE PIXTABLE DAR CORRECT RESIDUAL"
 #define MUSE_HDR_PT_DAR_CORR_C         "[Angstrom] ref. lambda for residual DAR correction"
+/** autocalibration checks */
+#define MUSE_HDR_PT_AUTOCAL_NAME       "ESO DRS MUSE PIXTABLE AUTOCAL"
+#define MUSE_HDR_PT_AUTOCAL_COMMENT    "%.3f (autocalibration clipping limit)"
 /** radial-velocity correction done, with the given velocity shift and reference */
 #define MUSE_HDR_PT_RVCORR             "ESO DRS MUSE PIXTABLE RVCORR"
 #define MUSE_HDR_PT_RVCORR_C           "[km/s] %scentric correction was applied"
@@ -315,6 +319,7 @@ unsigned short muse_pixtable_origin_get_slice(uint32_t);
 unsigned int muse_pixtable_origin_set_offset(muse_pixtable *, cpl_polynomial *, unsigned short, unsigned short);
 unsigned int muse_pixtable_origin_get_offset(muse_pixtable *, unsigned int, unsigned short, unsigned short);
 cpl_error_code muse_pixtable_origin_copy_offsets(muse_pixtable *, muse_pixtable *, unsigned int);
+cpl_error_code muse_pixtable_origin_decode_all(muse_pixtable *, unsigned short **, unsigned short **, unsigned char **, unsigned char **);
 unsigned int muse_pixtable_get_expnum(muse_pixtable *, cpl_size);
 
 /* the four main functions */
@@ -339,7 +344,7 @@ cpl_error_code muse_pixtable_restrict_wavelength(muse_pixtable *, double, double
 cpl_error_code muse_pixtable_restrict_xpos(muse_pixtable *, double, double);
 cpl_error_code muse_pixtable_restrict_ypos(muse_pixtable *, double, double);
 cpl_error_code muse_pixtable_erase_ifu_slice(muse_pixtable *, unsigned char, unsigned short);
-cpl_error_code muse_pixtable_and_selected_mask(muse_pixtable *, muse_mask *);
+cpl_error_code muse_pixtable_and_selected_mask(muse_pixtable *, const muse_mask *, const cpl_propertylist *, const cpl_table *);
 
 cpl_error_code muse_pixtable_dump(muse_pixtable *, cpl_size, cpl_size, unsigned char);
 muse_pixtable_wcs muse_pixtable_wcs_check(muse_pixtable *);
@@ -352,6 +357,8 @@ cpl_error_code muse_pixtable_reset_dq(muse_pixtable *, unsigned int);
 muse_imagelist *muse_pixtable_to_imagelist(muse_pixtable *);
 cpl_error_code muse_pixtable_from_imagelist(muse_pixtable *, muse_imagelist *);
 
+muse_pixtable *muse_pixtable_extract_wavelength(muse_pixtable *, double,
+                                                double);
 muse_pixtable **muse_pixtable_extracted_get_slices(muse_pixtable *);
 cpl_size muse_pixtable_extracted_get_size(muse_pixtable **);
 void  muse_pixtable_extracted_delete(muse_pixtable **);
