@@ -114,6 +114,7 @@ muse_postproc_properties_delete(muse_postproc_properties *aProp)
 #ifdef USE_LSF_PARAMS
   muse_lsf_params_delete_all(aProp->lsf_params);
 #endif
+  muse_table_delete(aProp->autocal_table);
   cpl_table_delete(aProp->raman_lines);
   cpl_table_delete(aProp->sky_lines);
   cpl_table_delete(aProp->sky_continuum);
@@ -516,11 +517,14 @@ muse_postproc_whitelight(muse_pixtable *aPixtable, double aCRSigma)
   @param    aWhite     output white-light image (optional)
   @return   CPL_ERROR_NONE on success, another CPL error code on failure.
 
-  A sky mask is used to select rows in the input pixel table that are then used
-  to compute the self-calibration. This sky mask is either taken as the input
-  SKY_MASK (component of aProp). In that case, its header must contain a proper
-  world-coordinate system. Or a mask is computed from the data itself, by
-  reconstructing and masking the white-light image of the field.
+  If the input aProp contains a autocal_table component, this is used to
+  directly apply the self-calibration and return.
+
+  Otherwise, a sky mask is used to select rows in the input pixel table that are
+  then used to compute the self-calibration. This sky mask is either taken as
+  the input SKY_MASK (component of aProp). In that case, its header must contain
+  a proper world-coordinate system. Or a mask is computed from the data itself,
+  by reconstructing and masking the white-light image of the field.
 
   In the latter case, the optional pointers aCube and aWhite will be filled,
   by the reconstructed cube and white-light image. Otherwise they remain
@@ -539,6 +543,13 @@ muse_postproc_run_autocalib(muse_pixtable *aPT, muse_postproc_properties *aProp,
 {
   cpl_ensure_code(aPT && aProp, CPL_ERROR_NULL_INPUT);
 
+  /* test if user autocalib factors table is provided */
+  if (aProp->autocal_table) {
+    /* yes: use that for self-calibration and return immediately */
+    cpl_error_code rc = muse_autocalib_apply(aPT, aProp->autocal_table);
+    return rc;
+  }
+
   /* spectra/pixel selection for self-calibration */
   cpl_size nrows = muse_pixtable_get_nrow(aPT);
   if (aProp->sky_mask) {
@@ -652,6 +663,10 @@ muse_postproc_run_autocalib(muse_pixtable *aPT, muse_postproc_properties *aProp,
          aProp->raman_width less than kMuseSliceSlitWidthA}
   @error{output message\, set and return CPL_ERROR_DATA_NOT_FOUND,
          aProp->raman_lines and/or aProp->lsf_cube are NULL}
+  @error{output message\, but return CPL_ERROR_NONE,
+         the pixel table does not contain data around either Raman feature}
+  @error{output warning\, try to do the correction\, but do not set error code,
+         the pixel table has partial coverage}
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code
@@ -659,6 +674,9 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
                             const cpl_table *aOffsets,
                             muse_datacube **aRamanCube)
 {
+  if (aRamanCube) {
+    *aRamanCube = NULL; /* NULL output input pointer for the failure cases */
+  }
   cpl_ensure_code(aPT && aProp, CPL_ERROR_NULL_INPUT);
   if (aProp->raman_width < kMuseSliceSlitWidthA) {
     cpl_msg_error(__func__, "Raman signal modelling not done, given width "
@@ -677,6 +695,19 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
     return cpl_error_set(__func__, CPL_ERROR_DATA_NOT_FOUND);
   }
 
+  /* central wavelengths of Raman peaks */
+  double wl[MUSE_RAMAN_RANGES] = {
+    6484., /* O2 */
+    6827.  /* N2 */
+  };
+  float llo = cpl_propertylist_get_float(aPT->header, MUSE_HDR_PT_LLO),
+        lhi = cpl_propertylist_get_float(aPT->header, MUSE_HDR_PT_LHI);
+  if (llo > wl[1] || lhi < wl[0]) {
+    cpl_msg_info(__func__, "Available wavelengths (%.3f ... %.3f) do not cover "
+                 "Raman features, not trying to correct them.", llo, lhi);
+    return CPL_ERROR_NONE;
+  }
+
   double cputime = cpl_test_get_cputime(),
          walltime = cpl_test_get_walltime();
 
@@ -698,10 +729,6 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
   cpl_msg_debug(__func__, "... done.");
 
   /* Create mini-pixtables containing just the wavelength range to fit. */
-  double wl[MUSE_RAMAN_RANGES] = {
-    6484., /* O2 */
-    6827.  /* N2 */
-  };
   muse_pixtable *pt[MUSE_RAMAN_RANGES];
   muse_image *images[MUSE_RAMAN_RANGES];
   muse_mask *sky_maskr[MUSE_RAMAN_RANGES];
@@ -710,6 +737,16 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
     double lo = wl[i] - aProp->raman_width * 0.5,
            hi = wl[i] + aProp->raman_width * 0.5;
     pt[i] = muse_pixtable_extract_wavelength(aPT, lo, hi);
+    if (!pt[i] || (pt[i] && muse_pixtable_get_nrow(pt[i]) == 0)) {
+      cpl_msg_warning(__func__, "Raman correction for %s feature not possible, "
+                      "%.3f..%.3f Angstrom is outside the wavelength range!",
+                      i == 0 ? "O2" : "N2", lo, hi);
+      muse_pixtable_delete(pt[i]);
+      pt[i] = NULL;
+      images[i] = NULL;
+      sky_maskr[i] = NULL;
+      continue;
+    }
 
     /* Create sky masks: one for each wavelength range */
     cpl_msg_info(__func__, "Intermediate resampling to produce white-light image"
@@ -718,6 +755,11 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
     cpl_msg_indent_more();
     images[i] = muse_postproc_whitelight(pt[i], aProp->skymodel_params.crsigmac);
     cpl_msg_indent_less();
+    if (!images[i]) {
+      /* should not occur, since the pixel table is checked above */
+      sky_maskr[i] = NULL;
+      continue;
+    }
     sky_maskr[i] = muse_sky_create_skymask(images[i], aProp->skymodel_params.ignore,
                                            aProp->skymodel_params.fraction,
                                            "ESO QC SCIPOST");
@@ -735,12 +777,32 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
     }
   } /* for i (both Raman ranges) */
 
+  /* check if any one of the two Raman features can be used at all */
+  if (!sky_maskr[0] && !sky_maskr[1]) {
+    /* no correction is possible, clean up and return */
+    cpl_msg_info(__func__, "Neither O2 nor N2 features can be analysed, not "
+                 "carrying out Raman correction.");
+    for (i = 0; i < MUSE_RAMAN_RANGES; i++) {
+      muse_image_delete(images[i]);
+      muse_pixtable_delete(pt[i]);
+    } /* for i (both Raman ranges) */
+    cpl_table_erase_column(aPT->table, "raman");
+    cpl_table_erase_column(aPT->table, "lsf");
+    return CPL_ERROR_NONE;
+  } /* no sky mask */
+
   /* Remove masked pixels of both ranges (and possibly an input sky mask) *
    * from both mini pixel tables, and then remove all bad pixels as well. */
   for (i = 0; i < MUSE_RAMAN_RANGES; i++) {
+    if (!pt[i] || !sky_maskr[i]) {
+      continue;
+    }
     int j;
     cpl_table_select_all(pt[i]->table);
     for (j = 0; j < MUSE_RAMAN_RANGES; j++) {
+      if (!pt[j] || !sky_maskr[j]) {
+        continue;
+      }
       muse_pixtable_and_selected_mask(pt[i], sky_maskr[j], NULL, NULL);
     }
     /* select pixels of an overall sky mask, possibly using its WCS */
@@ -765,6 +827,9 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
 
   /* Add the LSF column to each pixtable, and do the fit. */
   for (i = 0; i < MUSE_RAMAN_RANGES; i++) {
+    if (!pt[i]) {
+      continue;
+    }
     muse_raman_add_lsf(pt[i], aProp->raman_lines, i, 1.0, aProp->lsf_cube);
   } /* for i (both Raman ranges) */
 
@@ -818,7 +883,11 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
     /* Reconstruct an image of the Raman light distribution as well. */
     muse_image *fits[MUSE_RAMAN_RANGES];
     for (i = 0; i < MUSE_RAMAN_RANGES; i++) {
-      fits[i] = muse_raman_simulate_image(images[i], param);
+      if (!images[i]) {
+        fits[i] = NULL;
+      } else {
+        fits[i] = muse_raman_simulate_image(images[i], param);
+      }
     } /* for i (both Raman ranges) */
 
     /* Now, to be able to saving everything into one file, use a datacube. *
@@ -826,8 +895,13 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
      * units.                                                              */
     *aRamanCube = cpl_calloc(1, sizeof(muse_datacube));
     (*aRamanCube)->data = cpl_imagelist_new();
-    cpl_imagelist_set((*aRamanCube)->data, cpl_image_duplicate(fits[0]->data), 0);
-    (*aRamanCube)->header = cpl_propertylist_duplicate(fits[0]->header);
+    if (fits[0]) {
+      cpl_imagelist_set((*aRamanCube)->data, cpl_image_duplicate(fits[0]->data), 0);
+      (*aRamanCube)->header = cpl_propertylist_duplicate(fits[0]->header);
+    } else {
+      cpl_imagelist_set((*aRamanCube)->data, cpl_image_duplicate(fits[1]->data), 0);
+      (*aRamanCube)->header = cpl_propertylist_duplicate(fits[1]->header);
+    }
 
     /* write some QC parameters */
     cpl_propertylist_append_double((*aRamanCube)->header, QC_POST_RAMAN_SP_XX,
@@ -849,12 +923,19 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
      * convert the sky masks into images.                   */
     muse_image *masks[MUSE_RAMAN_RANGES];
     for (i = 0; i < MUSE_RAMAN_RANGES; i++) {
+      if (!sky_maskr[i]) {
+        masks[i] = NULL;
+        continue;
+      }
       masks[i] = muse_image_new();
       masks[i]->header = cpl_propertylist_duplicate(sky_maskr[i]->header);
       masks[i]->data = cpl_image_new_from_mask(sky_maskr[i]->mask);
       muse_mask_delete(sky_maskr[i]);
 
       /* Also scale both Raman images using the flux of the Raman lines. */
+      if (!fits[i]) {
+        continue;
+      }
       cpl_image_multiply_scalar(fits[i]->data, totflux[i]);
       cpl_propertylist_update_string(fits[i]->header, "BUNIT",
                                      kMuseFluxUnitString);
@@ -862,13 +943,18 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
 
     (*aRamanCube)->recimages = muse_imagelist_new();
     (*aRamanCube)->recnames = cpl_array_new(6, CPL_TYPE_STRING);
+    cpl_errorstate state = cpl_errorstate_get();
     muse_imagelist_set((*aRamanCube)->recimages, images[0], 0);
     muse_imagelist_set((*aRamanCube)->recimages, masks[0], 1);
     muse_imagelist_set((*aRamanCube)->recimages, images[1], 2);
     muse_imagelist_set((*aRamanCube)->recimages, masks[1], 3);
     muse_imagelist_set((*aRamanCube)->recimages, fits[0], 4);
     muse_imagelist_set((*aRamanCube)->recimages, fits[1], 5);
-
+    if (!cpl_errorstate_is_equal(state)) {
+      cpl_msg_debug(__func__, "Some images for the Raman cube were not saved...");
+      /* swallow the errors */
+      cpl_errorstate_set(state);
+    }
     cpl_array_set_string((*aRamanCube)->recnames, 0, "RAMAN_IMAGE_O2");
     cpl_array_set_string((*aRamanCube)->recnames, 1, "SKY_MASK_O2");
     cpl_array_set_string((*aRamanCube)->recnames, 2, "RAMAN_IMAGE_N2");
@@ -950,6 +1036,8 @@ muse_postproc_correct_raman(muse_pixtable *aPT, muse_postproc_properties *aProp,
          aProp or aProp->exposures are NULL}
   @error{set CPL_ERROR_ILLEGAL_INPUT\, return NULL,
          aIndex is larger than the last index in the aProp->exposures table}
+  @error{set CPL_ERROR_TYPE_MISMATCH but return valid object,
+         the pixel table is in an unknown state of astrometric calibration}
  */
 /*----------------------------------------------------------------------------*/
 void *
@@ -1045,14 +1133,14 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
     if (aProp->response) {
       rc = muse_flux_calibrate(pt, aProp->response, aProp->extinction,
                                aProp->telluric);
-      if (rc != CPL_ERROR_NONE && rc != CPL_ERROR_UNSUPPORTED_MODE) {
+      if (rc != CPL_ERROR_NONE && rc != CPL_ERROR_CONTINUE) {
         /* for a more serious error, output a warning at least *
          * (the data may still be usable)                      */
         cpl_msg_warning(__func__, "Flux calibration returned rc=%d: %s", rc,
                         cpl_error_get_message());
       } else {
-        /* for no error or unsupported mode (an already flux         *
-         * calibrated table!) a debug message seems more appropriate */
+        /* for no error or "continue" (an already flux calibrated *
+         * table!) a debug message seems more appropriate         */
         cpl_msg_debug(__func__, "Flux calibration returned rc=%d: %s", rc,
                       rc != CPL_ERROR_NONE ? cpl_error_get_message_default(rc) : "");
       }
@@ -1068,7 +1156,7 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
   muse_image *whitelight = NULL;
 
   /* run self calibration using partitioned per-slice sky spectra */
-  if (aProp->autocalib == MUSE_POSTPROC_AUTOCALIB_DEEPFIELD) {
+  if (aProp->autocalib != MUSE_POSTPROC_AUTOCALIB_NONE) {
     muse_postproc_run_autocalib(pt, aProp, aOffsets, aACalOut, &cube,
                                 &whitelight);
   } /* if autocalib */
@@ -1352,19 +1440,31 @@ muse_postproc_process_exposure(muse_postproc_properties *aProp,
   } /* if MUSE_POSTPROC_ASTROMETRY */
 
   if (aProp->astrometry) {
-    cpl_propertylist *wcs = aProp->wcs;
-    if (!wcs) {
-      /* create WCS, trying to find spaxel scale for the instrument mode */
-      const char *mode = muse_pfits_get_insmode(pt->header);
-      cpl_msg_warning(__func__, "Using default MUSE %cFM astrometry, output world"
-                      " coordinates will be inaccurate!", mode[0]);
-      wcs = muse_wcs_create_default(pt->header);
-    }
-    rc = muse_wcs_project_tan(pt, wcs);
-    cpl_msg_debug(__func__, "Astrometry correction returned rc=%d: %s", rc,
-                  rc != CPL_ERROR_NONE ? cpl_error_get_message() : "");
-    if (!aProp->wcs) { /* if this was not input, we need to free again */
-      cpl_propertylist_delete(wcs);
+    if (muse_pixtable_wcs_check(pt) == MUSE_PIXTABLE_WCS_PIXEL) {
+      cpl_propertylist *wcs = aProp->wcs;
+      if (!wcs) {
+        /* create WCS, trying to find spaxel scale for the instrument mode */
+        const char *mode = muse_pfits_get_insmode(pt->header);
+        cpl_msg_warning(__func__, "Using default MUSE %cFM astrometry, output world"
+                        " coordinates will be inaccurate!", mode[0]);
+        wcs = muse_wcs_create_default(pt->header);
+      }
+      rc = muse_wcs_project_tan(pt, wcs);
+      cpl_msg_debug(__func__, "Astrometry correction returned rc=%d: %s", rc,
+                    rc != CPL_ERROR_NONE ? cpl_error_get_message() : "");
+      if (!aProp->wcs) { /* if this was not input, we need to free again */
+        cpl_propertylist_delete(wcs);
+      }
+    } else if (muse_pixtable_wcs_check(pt) == MUSE_PIXTABLE_WCS_NATSPH) {
+      cpl_msg_info(__func__, "Pixel table already projected to TAN, skipping "
+                   "astrometric calibration...");
+    } else if (muse_pixtable_wcs_check(pt) == MUSE_PIXTABLE_WCS_CELSPH) {
+      cpl_msg_info(__func__, "Pixel table already positioned, skipping "
+                   "astrometric calibration...");
+    } else {
+      const char *msg = "Pixel table in unknown state of astrometric calibration!";
+      cpl_msg_error(__func__, "%s", msg);
+      cpl_error_set_message(__func__, CPL_ERROR_TYPE_MISMATCH, "%s", msg);
     }
   } /* if astrometry */
 
@@ -1511,7 +1611,7 @@ muse_postproc_cube_load_output_wcs(muse_processing *aProcessing)
       valid = valid && !strncmp(cpl_array_get_string(ctypes, 1), "DEC--TAN", 9);
     }
     /* need AWAV or AWAV-LOG for axis 3 */
-    if (valid && cpl_array_get_string(ctypes, 2)) {
+    if (valid && naxis == 3 && cpl_array_get_string(ctypes, 2)) {
       const char *ctype3 = cpl_array_get_string(ctypes, 2);
       valid = valid
             && (!strncmp(ctype3, "AWAV", 5) || !strncmp(ctype3, "AWAV-LOG", 9) ||
@@ -1735,15 +1835,15 @@ muse_postproc_qc_fwhm(muse_processing *aProcessing, muse_datacube *aCube)
   cpl_ensure_code(aProcessing && aCube, CPL_ERROR_NULL_INPUT);
 
   /* determine which FITS keyword prefix to take */
-  char prefix[KEYWORD_LENGTH] = "";
+  const char *prefix = "";
   if (!strncmp(aProcessing->name, "muse_scipost", 13)) {
-    strncpy(prefix, QC_POST_PREFIX_SCIPOST, KEYWORD_LENGTH);
+    prefix = QC_POST_PREFIX_SCIPOST;
   } else if (!strncmp(aProcessing->name, "muse_exp_combine", 17)) {
-    strncpy(prefix, QC_POST_PREFIX_EXPCOMBINE, KEYWORD_LENGTH);
+    prefix = QC_POST_PREFIX_EXPCOMBINE;
   } else if (!strncmp(aProcessing->name, "muse_standard", 14)) {
-    strncpy(prefix, QC_POST_PREFIX_STANDARD, KEYWORD_LENGTH);
+    prefix = QC_POST_PREFIX_STANDARD;
   } else if (!strncmp(aProcessing->name, "muse_astrometry", 16)) {
-    strncpy(prefix, QC_POST_PREFIX_ASTROMETRY, KEYWORD_LENGTH);
+    prefix = QC_POST_PREFIX_ASTROMETRY;
   } else {
     cpl_msg_info(__func__, "Recipe \"%s\" found, not generating QC1 keywords",
                  aProcessing->name);
