@@ -68,7 +68,6 @@ typedef struct {
 /* common functions (muse_sky_common.c) */
 cpl_table *muse_sky_spectrum_from_cube(muse_datacube *, const cpl_mask *);
 muse_mask *muse_sky_create_skymask(muse_image *, double, double, const char *);
-cpl_error_code muse_sky_mark_cosmic(cpl_table *aSpectrum, muse_pixtable *aPixtable, double);
 cpl_error_code muse_sky_save_continuum(muse_processing *, const cpl_table *, cpl_propertylist *);
 
 /* functions about lines (muse_sky_lines.c) */
@@ -88,10 +87,6 @@ cpl_table *muse_sky_continuum_load(muse_processing *);
 void muse_sky_qc_lines(cpl_propertylist *, cpl_table *, const char *);
 void muse_sky_qc_continuum(cpl_propertylist *, cpl_table *, const char *);
 
-/* functions about row-by-row sky subtraction (muse_sky_rowbyrow.c) */
-cpl_error_code muse_sky_subtract_rowbyrow_mask(muse_image *, cpl_table *);
-cpl_error_code muse_sky_subtract_rowbyrow(muse_image *, cpl_table *, float, unsigned int);
-
 /* functions about sky subtraction (muse_sky_subtract.c) */
 cpl_array *muse_sky_lines_spectrum(const cpl_array *, cpl_table *, const cpl_image *, const muse_wcs *);
 cpl_error_code muse_sky_subtract_lines(muse_pixtable *, cpl_table *, muse_lsf_cube **);
