@@ -29,20 +29,20 @@
 
 /*----------------------------------------------------------------------------*/
 /**
-   @defgroup muse_skysub       Sky subtraction
-
-   The sky subtraction consists of two steps: the calculation of the common
-   sky parameters (continuum spectrum and emission line fluxes),
-   and the actual subtraction of this spectrum applied to LSF parameters
-   from each slice. The communication between these two steps is realized with
-   the <tt>muse_sky_master</tt> structure.
-
-   As input for the master sky calculation a parameter file is used that
-   will be read into a <tt>muse_sky_lines</tt> structure.
-
-   @copydetails muse_sky_lines_oh_transitions_def
-
-   @copydetails muse_sky_lines_lines_def
+ * @defgroup muse_skysub       Sky subtraction
+ *
+ * The sky subtraction consists of two steps: the calculation of the common
+ * sky parameters (continuum spectrum and emission line fluxes),
+ * and the actual subtraction of this spectrum applied to LSF parameters
+ * from each slice. The communication between these two steps is realized with
+ * the <tt>muse_sky_master</tt> structure.
+ *
+ * As input for the master sky calculation a parameter file is used that
+ * will be read into a <tt>muse_sky_lines</tt> structure.
+ *
+ * @copydetails muse_sky_lines_oh_transitions_def
+ *
+ * @copydetails muse_sky_lines_lines_def
  */
 /*----------------------------------------------------------------------------*/
 /**@{*/
@@ -162,13 +162,12 @@ muse_sky_subtract_lines(muse_pixtable *aPixtable,
   cpl_size n_slices = muse_pixtable_extracted_get_size(slice_pixtable);
   cpl_size i_slice;
   cpl_error_code rc[n_slices];
-  cpl_msg_info(__func__,
-               "Starting sky subtraction of %"CPL_SIZE_FORMAT" slices",
-               n_slices);
-#if 1
-#pragma omp parallel for default(none)                        \
-  shared(aLsfCube, aLines, n_slices, slice_pixtable, rc)
-#endif
+  cpl_msg_info(__func__, "Starting sky subtraction of %"CPL_SIZE_FORMAT
+               " slices", n_slices);
+  cpl_boolean debug = getenv("MUSE_DEBUG_SKY")
+                    && atoi(getenv("MUSE_DEBUG_SKY")) > 0;
+  #pragma omp parallel for default(none)                                       \
+          shared(aLsfCube, aLines, debug, n_slices, slice_pixtable, rc)
   for (i_slice = 0; i_slice < n_slices; i_slice++) {
     muse_pixtable *slice_pt = slice_pixtable[i_slice];
       uint32_t origin =
@@ -186,11 +185,10 @@ muse_sky_subtract_lines(muse_pixtable *aPixtable,
     }
 
     cpl_size nrows = muse_pixtable_get_nrow(slice_pt);
-    cpl_msg_debug(__func__,
-                  "Sky subtraction of %"CPL_SIZE_FORMAT
-                  " pixels for slice #%i.%i (%i)",
-      nrows, ifu, slice, (int)i_slice+1);
-
+    if (debug) {
+      cpl_msg_debug(__func__, "Sky subtraction of %"CPL_SIZE_FORMAT" pixels for"
+                    " slice #%i.%i (%i)", nrows, ifu, slice, (int)i_slice+1);
+    }
     cpl_image *lsfImage = cpl_imagelist_get(aLsfCube[ifu-1]->img, slice-1);
     rc[i_slice] = muse_sky_subtract_lines_slice(slice_pt,
                                                 aLines, lsfImage,
@@ -219,8 +217,8 @@ muse_sky_subtract_continuum(muse_pixtable *aPixtable, cpl_table *aSpectrum)
 {
   cpl_ensure_code(aPixtable != NULL, CPL_ERROR_NULL_INPUT);
   cpl_ensure_code(aPixtable->table != NULL, CPL_ERROR_NULL_INPUT);
-  cpl_ensure_code(muse_cpltable_check(aPixtable->table, muse_pixtable_def) == CPL_ERROR_NONE,
-                  CPL_ERROR_DATA_NOT_FOUND);
+  cpl_ensure_code(muse_cpltable_check(aPixtable->table, muse_pixtable_def)
+                  == CPL_ERROR_NONE, CPL_ERROR_DATA_NOT_FOUND);
   cpl_ensure_code(aSpectrum != NULL, CPL_ERROR_NULL_INPUT);
   cpl_ensure_code(cpl_table_has_column(aSpectrum, "lambda") &&
                   cpl_table_has_column(aSpectrum, "flux"),
@@ -233,51 +231,14 @@ muse_sky_subtract_continuum(muse_pixtable *aPixtable, cpl_table *aSpectrum)
                "subtraction (range of continuum)", lmin, lmax);
   muse_pixtable_restrict_wavelength(aPixtable, lmin, lmax);
 
-  muse_pixtable **slice_pixtable =
-    muse_pixtable_extracted_get_slices(aPixtable);
-  cpl_size n_slices = muse_pixtable_extracted_get_size(slice_pixtable);
-  cpl_size i_slice;
-#if 1
-#pragma omp parallel for default(none)                        \
-  shared(aSpectrum, n_slices, slice_pixtable)
-#endif
-  for (i_slice = 0; i_slice < n_slices; i_slice++) {
-    muse_pixtable *slice_pt = slice_pixtable[i_slice];
-    cpl_propertylist *order = cpl_propertylist_new();
-    cpl_propertylist_append_bool(order, MUSE_PIXTABLE_LAMBDA, CPL_FALSE);
-    cpl_table_sort(slice_pt->table, order);
-    cpl_propertylist_delete(order);
-    cpl_array *lambda = NULL;
-    if (cpl_table_get_column_type(slice_pt->table, MUSE_PIXTABLE_LAMBDA)
-        == CPL_TYPE_DOUBLE) {
-      lambda = muse_cpltable_extract_column(slice_pt->table,
-                                            MUSE_PIXTABLE_LAMBDA);
-    } else {
-      cpl_table_cast_column(slice_pt->table, MUSE_PIXTABLE_LAMBDA,
-                            "lambda_double", CPL_TYPE_DOUBLE);
-      lambda = muse_cpltable_extract_column(slice_pt->table, "lambda_double");
-    }
+  cpl_array *lambda = muse_cpltable_extract_column(aSpectrum, "lambda"),
+            *flux = muse_cpltable_extract_column(aSpectrum, "flux");
+  muse_pixtable_spectrum_apply(aPixtable, lambda, flux,
+                               MUSE_PIXTABLE_OPERATION_SUBTRACT);
+  cpl_array_unwrap(lambda);
+  cpl_array_unwrap(flux);
 
-    cpl_array *data = muse_cpltable_extract_column(slice_pt->table,
-                                                   MUSE_PIXTABLE_DATA);
-    cpl_array *continuum
-      = muse_cplarray_interpolate_table_linear(lambda, aSpectrum,
-                                               "lambda", "flux");
-
-    cpl_msg_debug(__func__, "Subtracting continuum from %"CPL_SIZE_FORMAT
-                  " pixels",
-                 cpl_array_get_size(continuum));
-
-    cpl_array_subtract(data, continuum);
-    cpl_array_delete(continuum);
-    cpl_array_unwrap(data);
-    cpl_array_unwrap(lambda);
-    if (cpl_table_has_column(slice_pt->table, "lambda_double")) {
-      cpl_table_erase_column(slice_pt->table, "lambda_double");
-    }
-  }
-  muse_pixtable_extracted_delete(slice_pixtable);
   return CPL_ERROR_NONE;
-}
+} /* muse_sky_subtract_continuum() */
 
 /**@}*/
