@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2018 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -105,6 +105,18 @@ typedef enum {
    MUSE_UTILS_CENTROID_MEDIAN      /**< subtract median value as background */
 } muse_utils_centroid_type;
 
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief Type of spectrum smoothing to use (e.g. for the response curve)
+ */
+/*----------------------------------------------------------------------------*/
+typedef enum {
+  MUSE_SPECTRUM_SMOOTH_NONE = 0, /**< no smoothing at all! */
+  MUSE_SPECTRUM_SMOOTH_MEDIAN, /**< sliding median of +/- 15 Angstrom width */
+  MUSE_SPECTRUM_SMOOTH_PPOLY /**< piecewise cubic polynomial plus *
+                              *   median of +/- 15 Angstrom width */
+} muse_spectrum_smooth_type;
+
 /**@}*/
 
 /*----------------------------------------------------------------------------*
@@ -128,6 +140,7 @@ cpl_error_code muse_utils_filter_copy_properties(cpl_propertylist *, const muse_
 
 char *muse_utils_header_get_lamp_names(cpl_propertylist *, char);
 cpl_array *muse_utils_header_get_lamp_numbers(cpl_propertylist *);
+cpl_matrix *muse_matrix_new_gaussian_2d(int, int, double);
 cpl_image *muse_utils_image_fit_polynomial(const cpl_image *, unsigned short, unsigned short);
 cpl_error_code muse_utils_image_get_centroid_window(cpl_image *, int, int, int, int, double *, double *, muse_utils_centroid_type);
 cpl_error_code muse_utils_get_centroid(const cpl_matrix *, const cpl_vector *, const cpl_vector *, double *, double *, muse_utils_centroid_type);
@@ -135,7 +148,7 @@ cpl_error_code muse_utils_fit_multigauss_1d(const cpl_vector *, const cpl_bivect
 cpl_error_code muse_utils_fit_moffat_2d(const cpl_matrix *, const cpl_vector *, const cpl_vector *, cpl_array *, cpl_array *, const cpl_array *, double *, double *);
 cpl_polynomial *muse_utils_iterate_fit_polynomial(cpl_matrix *, cpl_vector *, cpl_vector *, cpl_table *, const unsigned int, const double, double *, double *);
 double muse_utils_pixtable_fit_line_gaussian(muse_pixtable *, double, double, double, float, float, unsigned char, cpl_array *, cpl_array *);
-cpl_matrix *muse_matrix_new_gaussian_2d(int, int, double);
+cpl_error_code muse_utils_spectrum_smooth(muse_table *, muse_spectrum_smooth_type);
 
 cpl_error_code muse_utils_copy_modified_header(cpl_propertylist *, cpl_propertylist *, const char *, const char *);
 cpl_error_code muse_utils_set_hduclass(cpl_propertylist *, const char *, const char *, const char *, const char *);
