@@ -528,12 +528,14 @@ muse_xcombine_tables(muse_pixtable **aPixtables, const cpl_table *aOffsets)
             ? muse_xcombine_find_offsets(aOffsets,
                                          muse_pfits_get_dateobs(aPixtables[i]->header))
             : NULL;
+    cpl_boolean offcor = CPL_FALSE; /* if any offset correction was applied */
     if (offsets) {
       if (isfinite(offsets[0]) && isfinite(offsets[1])) {
         ra -= offsets[0];
         dec -= offsets[1];
         cpl_msg_debug(__func__, "Applying coordinate offsets to exposure %d: "
                       "%e/%e deg", i + 1, offsets[0], offsets[1]);
+        offcor = CPL_TRUE;
         /* store in the header of the output pixel table */
         snprintf(keyword, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA, i + 1);
         snprintf(comment, KEYWORD_LENGTH, MUSE_HDR_OFFSETi_DRA_C, offsets[0] * 3600.);
@@ -586,10 +588,11 @@ muse_xcombine_tables(muse_pixtable **aPixtables, const cpl_table *aOffsets)
 
     /* compute simple (inaccurate!) offset for information */
     double avdec = (dec + dec0) / 2.,
-           raoff = (ra - ra0) * cos(avdec * CPL_MATH_RAD_DEG),
-           decoff = dec - dec0;
-    cpl_msg_info(__func__, "Approx. offset of exposure %u: %.3e,%.3e deg", i+1,
-                 raoff, decoff);
+           raoff = (ra - ra0) * cos(avdec * CPL_MATH_RAD_DEG) * 3600.,
+           decoff = (dec - dec0) * 3600.;
+    cpl_msg_info(__func__, "Distance of exposure %u (relative to exp. 1): "
+                 "%.1f,%.1f arcsec%s", i+1, raoff, decoff,
+                 offcor ? " (corrected offset)" : "");
 
     /* append the next pixel table to the end and delete the original */
     cpl_size nrow = muse_pixtable_get_nrow(pt);
