@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2018 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -157,10 +157,8 @@ muse_astrometry_compute(muse_processing *aProcessing,
                                                     MUSE_TAG_ASTROMETRY_WCS);
     cpl_free(object);
     if (rc != CPL_ERROR_NONE) {
-      for ( ; i < nexposures; i++) {
-        muse_wcs_object_delete(wcsobjs[i]);
-      } /* for rest of i */
-      break;
+      cpl_msg_warning(__func__, "Failed to save %s for exposure %d: %s",
+                      MUSE_TAG_ASTROMETRY_WCS, i+1, cpl_error_get_message());
     } /* if */
 
     /* XXX add option to create cube using the derived calibration *
