@@ -134,6 +134,7 @@ muse_utils_replace_nan(muse_image *aImage, float aValue)
   return CPL_ERROR_NONE;
 }
 
+
 /* XXX: Moved to muse_utils, kept here disabled until final place is found */
 #if 0
 /*----------------------------------------------------------------------------*/
@@ -330,9 +331,18 @@ muse_processing_fov_load_all(muse_processing *aProcessing)
 static cpl_boolean
 muse_align_check_detection_params(muse_exp_align_params_t *aParams)
 {
-  if (aParams->threshold <= 0.) {
+  if (aParams->bkgignore < 0.) {
     cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_INPUT,
-                          "Detection threshold must be greater than zero!");
+                          "Fraction of image pixel to ignore for the "
+                          "background estimation must be greater than"
+                          "or equal to zero!");
+    return CPL_FALSE;
+  }
+  if (aParams->bkgfraction <= 0.) {
+    cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_INPUT,
+                          "Fraction of image pixels to use for the "
+                          "background estimation must be greater "
+                          "than zero!");
     return CPL_FALSE;
   }
   if (aParams->fwhm < 0.5) {
@@ -430,7 +440,7 @@ muse_align_parse_valuelist(const char *aValuelist)
 /*----------------------------------------------------------------------------*/
 static int
 muse_align_celestial_from_pixel(cpl_table *aTable,
-                                    cpl_propertylist *aWCSHeader)
+                                cpl_propertylist *aWCSHeader)
 {
   muse_wcs *wcs = muse_wcs_new(aWCSHeader);
 
@@ -832,6 +842,69 @@ muse_cplmatrix_cosine(cpl_matrix *aMatrix)
   return CPL_ERROR_NONE;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @private
+  @brief    Estimate the background level of an exposure and its uncertainty.
+  @param    aImage        The image to analyse.
+  @param    aMinFraction  Matrix of declination offsets.
+  @param    aFraction     Matrix of mean declination.
+  @param    bkgValue      Optional, additional right ascension offset.
+  @param    bkgError      Optional, additional right ascension offset.
+  @return   A matrix containing the computed distances, or NULL.
+
+  The function computes for each matrix element the angular distance from
+  the given position and differences in right ascension and declination.
+  Extra shift parameters may be given. They are completely ignored if they are
+  set to NULL, otherwise they are applied as extra offsets in the computation.
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_error_code
+muse_align_estimate_background(muse_image *aImage,
+                               double aMinFraction, double aFraction,
+                               double *bkgValue, double *bkgError)
+{
+
+  cpl_ensure(aImage, CPL_ERROR_NULL_INPUT, CPL_ERROR_NULL_INPUT);
+  cpl_ensure((aImage->data && aImage->header),
+             CPL_ERROR_ILLEGAL_INPUT, CPL_ERROR_ILLEGAL_INPUT);
+
+  muse_image *image = muse_image_new();
+
+  image->data = cpl_image_duplicate(aImage->data);
+  image->header = cpl_propertylist_duplicate(aImage->header);
+
+
+  /* A FOV image may have pixels set to NaN, which should be ignored *
+   * for estimating the background. Also set an empty DQ image so    *
+   * that muse_sky_create_skymask() does not raise an error.         */
+  cpl_image_reject_value(image->data, CPL_VALUE_NOTFINITE);
+  image->dq = cpl_image_new(cpl_image_get_size_x(image->data),
+                            cpl_image_get_size_y(image->data),
+                            CPL_TYPE_INT);
+  muse_mask *mask = muse_sky_create_skymask(image, aMinFraction, aFraction,
+                                            "ESO QC EXPALIGN BKG");
+  if (!mask) {
+    cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_OUTPUT,
+                          "Creating background mask failed!");
+    return CPL_ERROR_ILLEGAL_OUTPUT;
+  }
+
+  cpl_mask_not(mask->mask); /* invert mask to reject non-sky pixels */
+  cpl_image_reject_from_mask(image->data, mask->mask);
+  muse_mask_delete(mask);
+
+  double bkg_sigma = 0.;
+  double bkg = cpl_image_get_mad(image->data, &bkg_sigma);
+
+  muse_image_delete(image);
+
+  *bkgValue = bkg;
+  *bkgError = bkg_sigma;
+
+  return CPL_ERROR_NONE;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @private
@@ -1085,6 +1158,7 @@ muse_align_update_offsets(double *aOffsetRA, double *aOffsetDEC, double *aWeight
   @brief    Compute field offsets for a set of exposures
   @param    aImagelist     the list of FOV images
   @param    aCataloglist   the list of sources for each FOV image.
+  @param    aFieldId       list of field identifiers.
   @param    aSearchradius  list of search radii
   @param    aNbins         number of histogram bins for first iteration
   @param    aWeight        Turn weighting on and off
@@ -1097,13 +1171,17 @@ muse_align_update_offsets(double *aOffsetRA, double *aOffsetDEC, double *aWeight
 /*----------------------------------------------------------------------------*/
 static cpl_table *
 muse_align_compute_field_offsets(muse_imagelist *aImagelist,
-                                 cpl_table **aCataloglist,
+                                 muse_table **aCataloglist,
+                                 const cpl_array *aFieldId,
                                  const cpl_array *aSearchradius,
                                  int aNbins, cpl_boolean aWeight,
                                  cpl_propertylist *aHeader)
 {
-  cpl_ensure(aImagelist && aCataloglist, CPL_ERROR_NULL_INPUT, NULL);
+  cpl_ensure(aImagelist && aCataloglist && aFieldId,
+             CPL_ERROR_NULL_INPUT, NULL);
   cpl_ensure(aSearchradius, CPL_ERROR_NULL_INPUT, NULL);
+  cpl_ensure(cpl_array_get_size(aFieldId) == muse_imagelist_get_size(aImagelist),
+             CPL_ERROR_ILLEGAL_INPUT, NULL);
   cpl_ensure(aNbins > 1, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
   cpl_size nfields = muse_imagelist_get_size(aImagelist);
@@ -1127,9 +1205,10 @@ muse_align_compute_field_offsets(muse_imagelist *aImagelist,
 
   /* record number of detections in each image as QC parameter */
   for (ifield = 0; ifield < nfields; ++ifield) {
-    char *kw = cpl_sprintf(QC_ALIGN_NDETi, (int)ifield + 1);
+    cpl_size kfield = cpl_array_get_cplsize(aFieldId, ifield, NULL);
+    char *kw = cpl_sprintf(QC_ALIGN_NDETi, (int)kfield + 1);
     cpl_propertylist_append_int(aHeader, kw,
-                                cpl_table_get_nrow(aCataloglist[ifield]));
+                                cpl_table_get_nrow(aCataloglist[ifield]->table));
     cpl_free(kw);
   }
 
@@ -1180,8 +1259,8 @@ muse_align_compute_field_offsets(muse_imagelist *aImagelist,
 
     for (ipair = 0; ipair < npairs; ++ipair) {
       /* Compute the distance between any 2 stars in the 2 fields */
-      const cpl_table *catalog1 = aCataloglist[combinations[ipair].first];
-      const cpl_table *catalog2 = aCataloglist[combinations[ipair].second];
+      const cpl_table *catalog1 = aCataloglist[combinations[ipair].first]->table;
+      const cpl_table *catalog2 = aCataloglist[combinations[ipair].second]->table;
 
       cpl_size nstars1 = cpl_table_get_nrow(catalog1);
       cpl_size nstars2 = cpl_table_get_nrow(catalog2);
@@ -1300,6 +1379,7 @@ muse_align_compute_field_offsets(muse_imagelist *aImagelist,
            nomatch = 0;
   for (ifield = 0; ifield < nfields; ++ifield) {
     muse_image *fov = muse_imagelist_get(aImagelist, ifield);
+    cpl_size field_id = cpl_array_get_cplsize(aFieldId, ifield, NULL);
     const char *timestamp = muse_pfits_get_dateobs(fov->header);
     double mjd = muse_pfits_get_mjdobs(fov->header);
     cpl_table_set_string(offsets, MUSE_OFFSETS_DATEOBS, ifield, timestamp);
@@ -1312,7 +1392,7 @@ muse_align_compute_field_offsets(muse_imagelist *aImagelist,
       cpl_table_set_double(offsets, MUSE_OFFSETS_DDEC, ifield, dec_offset / deg2as);
       /* record QC parameter */
       int nmedian = cpl_array_get_median(aoverlaps[ifield]);
-      char *kw = cpl_sprintf(QC_ALIGN_NMATCHi, (int)ifield + 1);
+      char *kw = cpl_sprintf(QC_ALIGN_NMATCHi, (int)field_id + 1);
       cpl_propertylist_append_int(aHeader, kw, nmedian);
       cpl_free(kw);
       if (nmedian < minmatch) {
@@ -1321,7 +1401,7 @@ muse_align_compute_field_offsets(muse_imagelist *aImagelist,
     } else {
       cpl_table_set_double(offsets, MUSE_OFFSETS_DRA, ifield, 0.);
       cpl_table_set_double(offsets, MUSE_OFFSETS_DDEC, ifield, 0.);
-      char *kw = cpl_sprintf(QC_ALIGN_NMATCHi, (int)ifield + 1);
+      char *kw = cpl_sprintf(QC_ALIGN_NMATCHi, (int)field_id + 1);
       cpl_propertylist_append_int(aHeader, kw, 0);
       cpl_free(kw);
       nomatch++;
@@ -1349,20 +1429,20 @@ muse_align_compute_field_offsets(muse_imagelist *aImagelist,
   double qcmean = cpl_table_get_column_mean(offsets, MUSE_OFFSETS_DRA) * 3600.;
   double qcsdev = cpl_table_get_column_stdev(offsets, MUSE_OFFSETS_DRA) * 3600.;
 
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_MIN, qcmin);
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_MAX, qcmax);
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_MEAN, qcmean);
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DRA_STDEV, qcsdev);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DRA_MIN, qcmin);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DRA_MAX, qcmax);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DRA_MEAN, qcmean);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DRA_STDEV, qcsdev);
 
   qcmin  = cpl_table_get_column_min(offsets, MUSE_OFFSETS_DDEC) * 3600.;
   qcmax  = cpl_table_get_column_max(offsets, MUSE_OFFSETS_DDEC) * 3600.;
   qcmean = cpl_table_get_column_mean(offsets, MUSE_OFFSETS_DDEC) * 3600.;
   qcsdev = cpl_table_get_column_stdev(offsets, MUSE_OFFSETS_DDEC) * 3600.;
 
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DDEC_MIN, qcmin);
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DDEC_MAX, qcmax);
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DDEC_MEAN, qcmean);
-  cpl_propertylist_append_double(aHeader, QC_ALIGN_DDEC_STDEV, qcsdev);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DDEC_MIN, qcmin);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DDEC_MAX, qcmax);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DDEC_MEAN, qcmean);
+  cpl_propertylist_append_float(aHeader, QC_ALIGN_DDEC_STDEV, qcsdev);
 
   return offsets;
 }
@@ -1441,6 +1521,7 @@ muse_exp_align_compute(muse_processing *aProcessing,
    * flag bad pixel. Replace them with an appropriate numerical value     */
   for (iexposure = 0; iexposure < nexposures; ++iexposure) {
     muse_image *image = muse_imagelist_get(fovimages, iexposure);
+    cpl_image_reject_value(image->data, CPL_VALUE_NOTFINITE);
     muse_utils_replace_nan(image, 0.);
   }
 
@@ -1451,14 +1532,31 @@ muse_exp_align_compute(muse_processing *aProcessing,
   double roundness[2] = {aParams->roundmin, aParams->roundmax};
   double sharpness[2] = {aParams->sharpmin, aParams->sharpmax};
 
-  cpl_table **srclists = cpl_calloc(nexposures, sizeof *srclists);
+  muse_table **srclists = cpl_calloc(nexposures, sizeof *srclists);
 
   for (iexposure = 0; iexposure < nexposures; ++iexposure) {
     cpl_msg_debug(__func__, "Detecting sources on image %" CPL_SIZE_FORMAT
                   " of %" CPL_SIZE_FORMAT, iexposure + 1, nexposures);
 
     muse_image *fov = muse_imagelist_get(fovimages, iexposure);
+
+    srclists[iexposure] = muse_table_new();
+
+    /* Estimate FOV sky background for use as detection threshold */
+    double bkg_sigma = 0.;
+    double bkg = 0.;
     double threshold = aParams->threshold;
+
+    muse_align_estimate_background(fov, aParams->bkgignore,
+                                   aParams->bkgfraction, &bkg, &bkg_sigma);
+
+    cpl_msg_info(__func__, "Median background level computed from sky mask: "
+                 "%.6f +/- %.6f (MAD)", bkg, bkg_sigma);
+
+    if (aParams->threshold <= 0.) {
+      threshold = bkg + fabs(threshold) * bkg_sigma;
+    }
+
     cpl_boolean iterate = CPL_TRUE;
     int iteration = 0;
     cpl_size ndetections = 0;
@@ -1466,7 +1564,7 @@ muse_exp_align_compute(muse_processing *aProcessing,
     cpl_image *image = cpl_image_cast(fov->data, CPL_TYPE_DOUBLE);
 
     /* If the image does not have an even number of pixels along the *
-     * any axis discard the last column and ro respectively.         */
+     * any axis discard the last column and row respectively.         */
 
     /* XXX: The size limit along the x- and y-axis corresponds to    *
      *      minimum convolution box size of muse_find_stars(). This  *
@@ -1476,7 +1574,7 @@ muse_exp_align_compute(muse_processing *aProcessing,
 
     if ((nx < 6) || (ny < 6)) {
       cpl_image_delete(image);
-      muse_vfree((void **)srclists, nexposures, (muse_free_func)cpl_table_delete);
+      muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
       muse_imagelist_delete(fovimages);
       cpl_msg_error(__func__, "Image %" CPL_SIZE_FORMAT " of %"
                     CPL_SIZE_FORMAT "is too small!", iexposure + 1, nexposures);
@@ -1495,7 +1593,7 @@ muse_exp_align_compute(muse_processing *aProcessing,
       cpl_image *_image = cpl_image_extract(image, 1, 1, nx, ny);
       if (!_image) {
         cpl_image_delete(image);
-        muse_vfree((void **)srclists, nexposures, (muse_free_func)cpl_table_delete);
+        muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
         muse_imagelist_delete(fovimages);
         cpl_msg_error(__func__, "Trimming image %" CPL_SIZE_FORMAT " failed",
                       iexposure + 1);
@@ -1508,15 +1606,19 @@ muse_exp_align_compute(muse_processing *aProcessing,
     while (iterate) {
       cpl_errorstate status = cpl_errorstate_get();
 
+      cpl_msg_info(__func__, "Searching for sources using pixel value %.6f "
+                   "as detection threshold", threshold);
+
       detections = muse_find_stars(image, threshold, aParams->fwhm,
                                    roundness, sharpness);
+
       if (!detections) {
         if (cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND) {
           cpl_errorstate_set(status);
           ndetections = 0;
         } else {
           cpl_image_delete(image);
-          muse_vfree((void **)srclists, nexposures, (muse_free_func)cpl_table_delete);
+          muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
           muse_imagelist_delete(fovimages);
           cpl_msg_error(__func__, "Source detection failed on image %"
                         CPL_SIZE_FORMAT " of %" CPL_SIZE_FORMAT, iexposure + 1,
@@ -1550,69 +1652,223 @@ muse_exp_align_compute(muse_processing *aProcessing,
     cpl_image_delete(image);
 
     if (!detections) {
-      muse_vfree((void **)srclists, nexposures, (muse_free_func)cpl_table_delete);
-      muse_imagelist_delete(fovimages);
       if (ndetections == 0) {
-        cpl_msg_error(__func__, "No point-sources were detected in image %"
-                      CPL_SIZE_FORMAT, iexposure + 1);
+        cpl_msg_warning(__func__, "No point-sources were detected in image %"
+                        CPL_SIZE_FORMAT, iexposure + 1);
       } else if (ndetections < nobjects[0]) {
-        cpl_msg_error(__func__, "Too few (%" CPL_SIZE_FORMAT ") point-sources "
-                      "were detected in image %" CPL_SIZE_FORMAT, ndetections,
-                      iexposure + 1);
+        cpl_msg_warning(__func__, "Too few (%" CPL_SIZE_FORMAT ") point-sources "
+                        "were detected in image %" CPL_SIZE_FORMAT, ndetections,
+                        iexposure + 1);
       } else if (ndetections > nobjects[1]) {
-        cpl_msg_error(__func__, "Too many (%" CPL_SIZE_FORMAT ") point-sources "
-                      "were detected in image %" CPL_SIZE_FORMAT, ndetections,
-                      iexposure + 1);
+        cpl_msg_warning(__func__, "Too many (%" CPL_SIZE_FORMAT ") point-sources "
+                        "were detected in image %" CPL_SIZE_FORMAT, ndetections,
+                        iexposure + 1);
       } else {
-        cpl_msg_error(__func__, "Invalid number (%" CPL_SIZE_FORMAT ") of "
-                      "point-sources were detected in image %" CPL_SIZE_FORMAT,
-                      ndetections, iexposure + 1);
+        cpl_msg_warning(__func__, "Invalid number (%" CPL_SIZE_FORMAT ") of "
+                        "point-sources were detected in image %" CPL_SIZE_FORMAT,
+                        ndetections, iexposure + 1);
       }
-      return -1;
-    } /* if no detections */
+      srclists[iexposure]->table = muse_cpltable_new(muse_source_list_def, 0);
+    } else {
+      cpl_msg_info(__func__, "%" CPL_SIZE_FORMAT " point-sources detected in "
+                   "image %" CPL_SIZE_FORMAT, ndetections, iexposure + 1);
 
-    cpl_msg_info(__func__, "%" CPL_SIZE_FORMAT " point-sources detected in "
-                 "image %" CPL_SIZE_FORMAT, ndetections, iexposure + 1);
+      /* Compute right ascension and declination from the pixel coordinates of *
+       * each detected point-source using the WCS information of each exposure */
+      if (muse_align_celestial_from_pixel(detections, fov->header)) {
+        muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
+        muse_imagelist_delete(fovimages);
+        cpl_msg_error(__func__, "Computing WCS coordinates from pixel positions "
+                      "failed for image %" CPL_SIZE_FORMAT " of %"
+                      CPL_SIZE_FORMAT, iexposure + 1, nexposures);
+        return -1;
+      }
 
-    /* Compute right ascension and declination from the pixel coordinates of *
-     * each detected point-source using the WCS information of each exposure */
-    if (muse_align_celestial_from_pixel(detections, fov->header)) {
-      muse_vfree((void **)srclists, nexposures, (muse_free_func)cpl_table_delete);
-      muse_imagelist_delete(fovimages);
-      cpl_msg_error(__func__, "Computing WCS coordinates from pixel positions "
-                    "failed for image %" CPL_SIZE_FORMAT " of %"
-                    CPL_SIZE_FORMAT, iexposure + 1, nexposures);
-      return -1;
+      cpl_size nsources = cpl_table_get_nrow(detections);
+      cpl_table *srclist = muse_cpltable_new(muse_source_list_def, nsources);
+
+      cpl_size isource;
+      for (isource = 0; isource < nsources; ++isource) {
+        double value = 0.;
+        int invalid = 0;
+
+        cpl_table_set_int(srclist, MUSE_SRCLIST_ID, isource, isource + 1);
+
+        value = cpl_table_get_double(detections, MUSE_SRCLIST_X,
+                                     isource, &invalid);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_X, isource, value);
+        value = cpl_table_get_double(detections, MUSE_SRCLIST_Y,
+                                     isource, &invalid);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_Y, isource, value);
+        value = cpl_table_get_double(detections, MUSE_SRCLIST_FLUX,
+                                     isource, &invalid);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_FLUX, isource, value);
+        value = cpl_table_get_double(detections, MUSE_SRCLIST_SHARPNESS,
+                                     isource, &invalid);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_SHARPNESS, isource, value);
+        value = cpl_table_get_double(detections, MUSE_SRCLIST_ROUNDNESS,
+                                     isource, &invalid);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_ROUNDNESS, isource, value);
+        value = cpl_table_get_double(detections, MUSE_SRCLIST_RA,
+                                     isource, &invalid);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_RA, isource, value);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_RACORR, isource, value);
+        value = cpl_table_get_double(detections, MUSE_SRCLIST_DEC,
+                                     isource, &invalid);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_DEC, isource, value);
+        cpl_table_set_double(srclist, MUSE_SRCLIST_DECCORR, isource, value);
+
+      }
+
+      srclists[iexposure]->table = srclist;
+      cpl_table_delete(detections);
     }
-    srclists[iexposure] = detections;
+
+    srclists[iexposure]->header = cpl_propertylist_duplicate(fov->header);
+
+    cpl_propertylist_erase_regexp(srclists[iexposure]->header, "^ESO QC.*",
+                                  CPL_FALSE);
+
+    cpl_propertylist_update_int(srclists[iexposure]->header,
+                                QC_ALIGN_NDET, ndetections);
+    cpl_propertylist_update_float(srclists[iexposure]->header,
+                                  QC_ALIGN_BKG_MEDIAN, bkg);
+    cpl_propertylist_update_float(srclists[iexposure]->header,
+                                  QC_ALIGN_BKG_MAD, bkg_sigma);
+    cpl_propertylist_update_float(srclists[iexposure]->header,
+                                  QC_ALIGN_THRESHOLD, threshold);
+
   } /* for iexposure */
 
 
   /* Compute exposure offsets */
   cpl_array *radii = muse_align_parse_valuelist(aParams->rsearch);
   if (!radii) {
-    muse_vfree((void **)srclists, nexposures, (muse_free_func)cpl_table_delete);
+    muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
     muse_imagelist_delete(fovimages);
     cpl_msg_error(__func__, "Cannot compute field offsets! No valid search "
                   "radius was given!");
     return -1;
   }
 
-  cpl_msg_info(__func__, "Calculating field offsets...");
+  /* XXX: Dealing with subsets of the input fields should be better *
+   *      integrated into the code structure, possibly done by      *
+   *      muse_align_compute_field_offsets().                       */
+
+  /* Before the field offsets are calculated the fields and the source *
+   * list with no detected sources have to be filtered out. They will  *
+   * be added again later when the final offset table is prepared.     *
+   * Their corresponding field offsets will be forced to zero.         */
+
+  muse_imagelist *_fovimages = muse_imagelist_new();
+  muse_table **_srclists = cpl_calloc(nexposures, sizeof *_srclists);
+  cpl_array *field_id = cpl_array_new(nexposures, CPL_TYPE_SIZE);
   cpl_propertylist *header = cpl_propertylist_new();
-  cpl_table *offsets = muse_align_compute_field_offsets(fovimages, srclists, radii,
-                                                        aParams->nbins, aParams->weight,
-                                                        header);
-  cpl_array_delete(radii);
-  muse_vfree((void **)srclists, nexposures, (muse_free_func)cpl_table_delete);
-  muse_imagelist_delete(fovimages);
-  if (!offsets) {
-    cpl_msg_error(__func__, "Could not compute FOV offsets for %"
-                  CPL_SIZE_FORMAT " images", nexposures);
+
+  cpl_size kexposures = 0;
+  for (iexposure = 0; iexposure < nexposures; ++iexposure) {
+    if (cpl_table_get_nrow(srclists[iexposure]->table) > 0) {
+      muse_imagelist_set(_fovimages, muse_imagelist_get(fovimages, iexposure),
+                         kexposures);
+      _srclists[kexposures] = srclists[iexposure];
+      cpl_array_set(field_id, kexposures, iexposure);
+      ++kexposures;
+    } else {
+      char *kw = cpl_sprintf(QC_ALIGN_NDETi, (int)iexposure + 1);
+      cpl_propertylist_append_int(header, kw, 0);
+      cpl_free(kw);
+
+      kw = cpl_sprintf(QC_ALIGN_NMATCHi, (int)iexposure + 1);
+      cpl_propertylist_append_int(header, kw, 0);
+      cpl_free(kw);
+
+      cpl_msg_warning(__func__, "No sources are available for image % "
+                      CPL_SIZE_FORMAT ". This field will be ignored and no "
+                      "coordinate offsets will be calculated!", iexposure + 1);
+    }
+  }
+  _srclists = cpl_realloc(_srclists, kexposures * sizeof *_srclists);
+  muse_cplarray_erase_invalid(field_id);
+
+  if (kexposures < 2) {
     cpl_propertylist_delete(header);
+    cpl_array_delete(field_id);
+    muse_vfree((void **)_srclists, kexposures, (muse_free_func)muse_table_delete);
+    muse_imagelist_delete(_fovimages);
+    muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
+    muse_imagelist_delete(fovimages);
+    cpl_msg_error(__func__, "Cannot compute field offsets! Source positions "
+                  "are only available for %" CPL_SIZE_FORMAT " out of %"
+                  CPL_SIZE_FORMAT " fields!", kexposures, nexposures);
     return -1;
   }
 
+  cpl_msg_info(__func__, "Calculating field offsets...");
+  cpl_table *offsets = muse_align_compute_field_offsets(_fovimages, _srclists,
+                                                        field_id, radii,
+                                                        aParams->nbins,
+                                                        aParams->weight,
+                                                        header);
+
+  /* XXX: Update the number of fields without matches only after *
+   *      calling muse_align_compute_field_offsets(). Otherwise  *
+   *      this value would be overwritten.                       */
+
+  int nomatch = cpl_propertylist_get_int(header, QC_ALIGN_NOMATCH) +
+      nexposures - kexposures;
+  cpl_propertylist_update_int(header, QC_ALIGN_NOMATCH, nomatch);
+
+
+  /* When cleaning up the filtered image list and the list of source lists *
+   * just clean up the container, as the actual data is managed by their   *
+   * full versions. The filtered data structures just served as a view for *
+   * the computation of the offsets.                                       */
+
+  cpl_free(_srclists);
+  iexposure = muse_imagelist_get_size(_fovimages);
+  while (iexposure) {
+    (void)muse_imagelist_unset(_fovimages, 0);
+    --iexposure;
+  }
+  muse_imagelist_delete(_fovimages);
+
+  cpl_array_delete(radii);
+  if (!offsets) {
+    cpl_propertylist_delete(header);
+    cpl_array_delete(field_id);
+    muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
+    muse_imagelist_delete(fovimages);
+    cpl_msg_error(__func__, "Could not compute FOV offsets for %"
+                  CPL_SIZE_FORMAT " images", nexposures);
+    return -1;
+  }
+
+  /* Finalize the source list product, and add columns with the corrected *
+   * source positions to the source lists.                                */
+  for (iexposure = 0; iexposure < cpl_array_get_size(field_id); ++iexposure) {
+    int invalid[2] = {0, 0};
+    double shift_ra = cpl_table_get_double(offsets, MUSE_OFFSETS_DRA,
+                                           iexposure, &invalid[0]);
+    double shift_dec = cpl_table_get_double(offsets, MUSE_OFFSETS_DDEC,
+                                            iexposure, &invalid[1]);
+
+    cpl_size ifield = cpl_array_get(field_id, iexposure, NULL);
+
+    if (invalid[0] || invalid[1]) {
+      cpl_msg_warning(__func__, "Invalid field offsets found for field %"
+                    CPL_SIZE_FORMAT ". No Correction will not be applied to "
+                    "the position of the detected sources.", ifield);
+    }
+    else {
+      cpl_table_subtract_scalar(srclists[ifield]->table, MUSE_SRCLIST_RACORR,
+                                shift_ra);
+      cpl_table_subtract_scalar(srclists[ifield]->table, MUSE_SRCLIST_DECCORR,
+                                shift_dec);
+    }
+  }
+
+  cpl_array_delete(field_id);
+
   /* Write offset table product to file */
   cpl_error_code rc = muse_processing_save_table(aProcessing, -1, offsets,
                                                  header, MUSE_TAG_OFFSET_LIST,
@@ -1620,6 +1876,18 @@ muse_exp_align_compute(muse_processing *aProcessing,
   cpl_propertylist_delete(header);
   cpl_table_delete(offsets);
 
+  /* Save source list products to file. Note that in order to get the *
+   * header information from FOV image associated to the source list  *
+   * product it has to be saved using CPL calls directly.             */
+  for (iexposure = 0; iexposure < nexposures; ++iexposure) {
+    muse_processing_save_table(aProcessing, -1, srclists[iexposure]->table,
+                               srclists[iexposure]->header,
+                               "SOURCE_LIST", MUSE_TABLE_TYPE_CPL);
+
+  }
+  muse_vfree((void **)srclists, nexposures, (muse_free_func)muse_table_delete);
+  muse_imagelist_delete(fovimages);
+
   if (rc != CPL_ERROR_NONE) {
     return -1;
   }
