@@ -275,8 +275,8 @@ muse_geometry_reconstruct_combined(muse_processing *aProcessing,
   #pragma omp parallel for default(none)                 /* as req. by Ralf */ \
           shared(gt, aParams, aProcessing, pts, skip)
   for (nifu = (unsigned)aParams->ifu1; nifu <= (unsigned)aParams->ifu2; nifu++) {
-    cpl_table *trace = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE, nifu),
-              *wavecal = muse_table_load(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
+    cpl_table *trace = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE, nifu),
+              *wavecal = muse_processing_load_ctable(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
     if (!trace || !wavecal) {
       cpl_table_delete(trace);
       cpl_table_delete(wavecal);
@@ -429,8 +429,8 @@ muse_geometry_reconstruct(muse_processing *aProcessing,
     #pragma omp parallel for default(none)               /* as req. by Ralf */ \
             shared(aParams, aProcessing, fn, frames, gt, pts)
     for (nifu = (unsigned)aParams->ifu1; nifu <= (unsigned)aParams->ifu2; nifu++) {
-      cpl_table *trace = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE, nifu),
-                *wavecal = muse_table_load(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
+      cpl_table *trace = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE, nifu),
+                *wavecal = muse_processing_load_ctable(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
       if (!trace || !wavecal) {
         cpl_table_delete(trace);
         cpl_table_delete(wavecal);
@@ -547,6 +547,7 @@ muse_geometry_reconstruct(muse_processing *aProcessing,
      * Use a trick so that the MASK_CHECK files appears first in the used *
      * frames and appear in the headers.                                  */
     muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_RAW, 1);
+#pragma omp critical(muse_processing_used_frames)
     cpl_frameset_insert(usedoriginal, cpl_frame_duplicate(frame)); /* also add here */
     cpl_propertylist *header = cpl_propertylist_load(fn, 0);
     muse_processing_save_cimage(aProcessing, -1, cpl_imagelist_get(cube->data, 0),
@@ -583,16 +584,14 @@ muse_geometry_compute(muse_processing *aProcessing,
   }
 
   /* load linelist upfront, to not having to do that in each thread */
-  cpl_table *linelist = muse_table_load(aProcessing, MUSE_TAG_LINE_CATALOG, 0);
-  cpl_propertylist *linehead = muse_propertylist_load(aProcessing,
-                                                      MUSE_TAG_LINE_CATALOG);
-  cpl_boolean listvalid = muse_wave_lines_check(linelist, linehead);
-  cpl_propertylist_delete(linehead);
-  cpl_vector *vlines = muse_geo_lines_get(linelist); /* suitable lines */
-  cpl_table_delete(linelist);
+  muse_table *linelist = muse_processing_load_table(aProcessing,
+                                                    MUSE_TAG_LINE_CATALOG, 0);
+  cpl_boolean listvalid = muse_wave_lines_check(linelist);
+  cpl_vector *vlines = muse_geo_lines_get(linelist->table); /* suitable lines */
+  muse_table_delete(linelist);
   if (!listvalid || !vlines) {
     cpl_msg_error(__func__, "%s could not be loaded/verified or enough suitable"
-                  "lines are missing", MUSE_TAG_LINE_CATALOG);
+                  " lines are missing", MUSE_TAG_LINE_CATALOG);
     cpl_vector_delete(vlines);
     return -1;
   }
@@ -614,8 +613,8 @@ muse_geometry_compute(muse_processing *aProcessing,
 
     /* load and check trace and wavecal tables early-on, *
      * to be able to quickly return on error             */
-    trace[nifu - 1] = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE, nifu);
-    wavecal[nifu - 1] = muse_table_load(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
+    trace[nifu - 1] = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE, nifu);
+    wavecal[nifu - 1] = muse_processing_load_ctable(aProcessing, MUSE_TAG_WAVECAL_TABLE, nifu);
     if (!trace[nifu - 1] || !wavecal[nifu - 1]) {
       cpl_msg_error(__func__, "Calibration could not be loaded for IFU %u: %s%s",
                     nifu, !trace[nifu - 1] ? " "MUSE_TAG_TRACE_TABLE : "",
