@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2015 European Southern Observatory
+ * Copyright (C) 2015-2016 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -353,8 +353,6 @@ muse_lingain_create_windowlist(const cpl_table *aTrace,
   cpl_ensure(aTrace && aExposureList && aList && aParams,
              CPL_ERROR_NULL_INPUT, NULL);
 
-  const unsigned short nslices = cpl_table_get_nrow(aTrace);
-
   cpl_table *windowlist = cpl_table_new(0);
   cpl_table_new_column(windowlist, "Quadrant", CPL_TYPE_INT);
   cpl_table_new_column(windowlist, "SliceNo", CPL_TYPE_INT);
@@ -378,11 +376,18 @@ muse_lingain_create_windowlist(const cpl_table *aTrace,
     cpl_size *qwindow = muse_quadrants_get_window(image, iquadrant);
 
     unsigned short islice;
-    for (islice = 1; islice <= nslices; ++islice) {
+    for (islice = 1; islice <= kMuseSlicesPerCCD; ++islice) {
+      cpl_errorstate _status = cpl_errorstate_get();
       cpl_polynomial **traces = muse_trace_table_get_polys_for_slice(aTrace,
                                                                      islice);
-      if (cpl_error_get_code() != CPL_ERROR_NONE) {
-        fprintf(stderr, "%d, %d, %d\n", (int)iquadrant, (int)islice, cpl_error_get_code());
+      /* If no tracing polynomial is found for the current slice index *
+       * issue a warning and continue with the next slice.             */
+      if (!traces) {
+        cpl_msg_warning(__func__, "slice %2d of IFU %hhu: tracing polynomials "
+                                  "missing!", islice, aParams->nifu);
+        muse_trace_polys_delete(traces);
+        cpl_errorstate_set(_status);
+        continue;
       }
       cpl_size torder = 0;
       double ledge = cpl_polynomial_get_coeff(traces[MUSE_TRACE_LEFT], &torder);
@@ -483,7 +488,8 @@ muse_lingain_quadrant_get_windows(cpl_table *aWindowList,
   @brief    Extract the data given by a list of window indices.
   @param    aGainTable  The gain table from which the data is extracted.
   @param    aWindows    The array of window indices to extract.
-  @param    aColumn     The name of the data column from which the data is taken.
+  @param    aColumn     The name of the data column from which the data is
+                        taken.
   @param    aRow        The row index from which the data is taken.
   @return   The array of extracted data values or NULL if an error occurs.
 
@@ -926,7 +932,8 @@ muse_lingain_quadrant_get_gain(muse_gain_fit_t *aGain,
   }
 
   /* Create the bins for the gain and the abscissa for the gain fits. */
-  cpl_size nbins = (aParams->signalmax - aParams->signalmin) / aParams->signalbin;
+  cpl_size nbins = (cpl_size)((aParams->signalmax - aParams->signalmin) /
+      aParams->signalbin + 0.5);
   cpl_array *gx = cpl_array_new(nbins, CPL_TYPE_DOUBLE);
   cpl_array *gy = cpl_array_new(nbins, CPL_TYPE_DOUBLE);
 
@@ -1110,11 +1117,11 @@ muse_lingain_quadrant_get_nonlinearity(muse_linearity_fit_t *aFit,
                                        muse_lingain_params_t *aParams)
 {
 
-  /* Build the log10(signal) grid. Using these signal bins determine *
-   * the set of measurement windows falling into each bin from the   *
-   * exposure with the longest exposure time. The selected windows   *
-   * are then used for the analysis of all exposure times
-   */
+  /* Build the log10(signal) grid. Using these signal bins to   *
+   * determine the set of measurement windows falling into each *
+   * bin from the exposure with the longest exposure time. The  *
+   * selected windows are then used for the analysis of all     *
+   * exposure times                                             */
 
   cpl_size ntimes = cpl_table_get_nrow(aGainTable);
   cpl_size irow;
@@ -1141,7 +1148,8 @@ muse_lingain_quadrant_get_nonlinearity(muse_linearity_fit_t *aFit,
   cpl_array_logarithm(sigref, 10.);
 
   cpl_size nsignal = cpl_array_get_size(sigref);
-  cpl_size nlevels = (aParams->ctsmax - aParams->ctsmin) / aParams->ctsbin;
+  cpl_size nlevels = (cpl_size)((aParams->ctsmax - aParams->ctsmin) /
+      aParams->ctsbin + 0.5);
 
   cpl_array **counts = cpl_calloc(nlevels, sizeof *counts);
   const double qgain = log10(aGain);
@@ -1235,8 +1243,8 @@ muse_lingain_quadrant_get_nonlinearity(muse_linearity_fit_t *aFit,
    * residuals are computed.                                             */
 
   /* Convert linear range limit to ADU */
-  double linmin = pow(10., aParams->linearmin) / aGain;
-  double linmax = pow(10., aParams->linearmax) / aGain;
+  double linmin = pow(10., aParams->linearmin - qgain);
+  double linmax = pow(10., aParams->linearmax - qgain);
 
   cpl_array **cresidual = cpl_malloc(klevel * sizeof *cresidual);
 
@@ -1354,15 +1362,19 @@ muse_lingain_quadrant_get_nonlinearity(muse_linearity_fit_t *aFit,
 
   /* Create signal bins for which the mean residual will be computed. *
    * Reuse the bin settings used for the gain computation.            */
-  cpl_size nbins = (aParams->signalmax - aParams->signalmin) / aParams->signalbin;
+  double signalmax = aParams->signalmax;
+  double signalmin = aParams->signalmin;
+  double signalbin = aParams->signalbin;
+
+  cpl_size nbins = (cpl_size)((signalmax - signalmin) / signalbin + 0.5);
   cpl_array *lx  = cpl_array_new(nbins, CPL_TYPE_DOUBLE);
   cpl_array *ly  = cpl_array_new(nbins, CPL_TYPE_DOUBLE);
   cpl_array *lyse = cpl_array_new(nbins, CPL_TYPE_DOUBLE);
 
   cpl_size ibin;
   for (ibin = 0; ibin < nbins; ++ibin) {
-    double bmin = aParams->signalmin + ibin * aParams->signalbin;
-    double bmax = bmin + aParams->signalbin;
+    double bmin = signalmin + ibin * signalbin;
+    double bmax = bmin + signalbin;
     double bmid = 0.5 * (bmin + bmax);
 
     cpl_array_set_double(lx, ibin, bmid);
@@ -1439,7 +1451,7 @@ muse_lingain_quadrant_get_nonlinearity(muse_linearity_fit_t *aFit,
     return CPL_ERROR_INCOMPATIBLE_INPUT;
   }
 
-  /* One extra data point is required so that residuals could be computed! */
+  /* One extra data point is required so that the residuals can be computed! */
   if (nx < aParams->order + 2) {
     cpl_msg_debug(__func__, "Fitting the non-linearity relation failed. "
                   "Insufficient data points; cannot fit a %d order "
@@ -1719,8 +1731,8 @@ muse_lingain_compute(muse_processing *aProcessing,
   }
 
   /* Load trace table */
-  cpl_table *trace = muse_table_load(aProcessing, MUSE_TAG_TRACE_TABLE,
-                                     aParams->nifu);
+  cpl_table *trace = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE,
+                                                 aParams->nifu);
 
   if (!trace) {
     cpl_msg_error(__func__, "%s could not be loaded!", MUSE_TAG_TRACE_TABLE);
@@ -1889,8 +1901,8 @@ muse_lingain_compute(muse_processing *aProcessing,
     cpl_propertylist_set_comment(header, keyword,
                                  "Order of the polynomial fit");
 
-    const cpl_array *coefficients = cpl_table_get_array(linearity, "Coefficients",
-                                                        iquadrant);
+    const cpl_array *coefficients =
+        cpl_table_get_array(linearity, "Coefficients", iquadrant);
     int icoefficient;
     for (icoefficient = 0; icoefficient < ncoefficient; ++icoefficient) {
       double c = cpl_array_get_double(coefficients, icoefficient, NULL);
