@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2014 European Southern Observatory
+ * Copyright (C) 2014-2018 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@
  *                             Functions code                                *
  *---------------------------------------------------------------------------*/
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Create the LSF of all slices of one IFU.
    @param aPixtable   special pixel table with arc lines
@@ -54,6 +55,7 @@
 
    This loops over all slices and calls muse_lsf_fit_slice() for each.
  */
+/*----------------------------------------------------------------------------*/
 static muse_lsf_cube *
 muse_lsf_compute_slices(muse_pixtable *aPixtable, muse_lsf_params_t *aParams)
 {
@@ -65,8 +67,8 @@ muse_lsf_compute_slices(muse_pixtable *aPixtable, muse_lsf_params_t *aParams)
   muse_pixtable **slice_pixtable = muse_pixtable_extracted_get_slices(aPixtable);
 
   int i_slice, n_slices = muse_pixtable_extracted_get_size(slice_pixtable);
-#pragma omp parallel for default(none)               /* as req. by Ralf */ \
-  shared(lsfCube, n_slices, slice_pixtable, aParams)
+  #pragma omp parallel for default(none)                 /* as req. by Ralf */ \
+          shared(lsfCube, n_slices, slice_pixtable, aParams)
   for (i_slice = 0; i_slice < n_slices; i_slice++) {
     cpl_image *img = cpl_imagelist_get(lsfCube->img, i_slice);
     muse_lsf_fit_slice(slice_pixtable[i_slice], img,
@@ -74,8 +76,9 @@ muse_lsf_compute_slices(muse_pixtable *aPixtable, muse_lsf_params_t *aParams)
   }
   muse_pixtable_extracted_delete(slice_pixtable);
   return lsfCube;
-}
+} /* muse_lsf_compute_slices() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Subtract the LSF from an arc pixtable
    @param aLsfCube    LSF datacube
@@ -85,13 +88,15 @@ muse_lsf_compute_slices(muse_pixtable *aPixtable, muse_lsf_params_t *aParams)
    pixel tables. The pixtable needs the additional columns
    <tt>line_lambda</tt> and <tt>line_flux</tt> to work.
  */
+/*----------------------------------------------------------------------------*/
 static cpl_error_code
-muse_lsf_subtract_arcs(muse_lsf_cube *aLsfCube, cpl_table *aPixtable) {
+muse_lsf_subtract_arcs(muse_lsf_cube *aLsfCube, cpl_table *aPixtable)
+{
   cpl_size n_rows = cpl_table_get_nrow(aPixtable);
   cpl_size i_row;
   for (i_row = 0; i_row < n_rows; i_row++) {
-    uint32_t origin = (uint32_t)cpl_table_get_int(aPixtable, MUSE_PIXTABLE_ORIGIN,
-                                                  i_row, NULL);
+    uint32_t origin = cpl_table_get_int(aPixtable, MUSE_PIXTABLE_ORIGIN, i_row,
+                                        NULL);
     int i_slice = muse_pixtable_origin_get_slice(origin);
     double lambda = cpl_table_get(aPixtable, "line_lambda", i_row, NULL);
     double flux = cpl_table_get(aPixtable, "line_flux", i_row, NULL);
@@ -106,8 +111,9 @@ muse_lsf_subtract_arcs(muse_lsf_cube *aLsfCube, cpl_table *aPixtable) {
                   - v * flux);
   }
   return CPL_ERROR_NONE;
-}
+} /* muse_lsf_subtract_arcs() */
 
+/*----------------------------------------------------------------------------*/
 /**
    @brief Create a header with the QC parameters for the LSF.
    @param aLsfCube    Computed LSF cube
@@ -123,6 +129,7 @@ muse_lsf_subtract_arcs(muse_lsf_cube *aLsfCube, cpl_table *aPixtable) {
 
    They are added to the given header.
  */
+/*----------------------------------------------------------------------------*/
 static cpl_error_code
 muse_lsf_qc(muse_lsf_cube *aLsfCube)
 {
@@ -178,33 +185,46 @@ muse_lsf_qc(muse_lsf_cube *aLsfCube)
   }
 
   return CPL_ERROR_NONE;
-}
+} /* muse_lsf_qc() */
 
 #ifdef USE_LSF_PARAMS
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Copmpute the old LSF parameters.
+  @param    aLines      arc line table
+  @param    aPixtable   the pixel table with the arc data
+  @param    aMaxiter    maximum number of iterations
+  @return   Array with LSF parameters.
+ */
+/*----------------------------------------------------------------------------*/
 muse_lsf_params **
-muse_lsf_params_compute(cpl_table *aLines,
-                        muse_pixtable *aPixtable,
-                        int max_iterations)
+muse_lsf_params_compute(cpl_table *aLines, muse_pixtable *aPixtable,
+                        int aMaxIter)
 {
   muse_pixtable **slice_pixtable = muse_pixtable_extracted_get_slices(aPixtable);
   int n_slices = muse_pixtable_extracted_get_size(slice_pixtable);
   int i_slice;
 
-  muse_lsf_params **lsfParams
-    = cpl_calloc(n_slices+1, sizeof(muse_lsf_params *));
+  muse_lsf_params **lsfParams = cpl_calloc(n_slices + 1,
+                                           sizeof(muse_lsf_params *));
   #pragma omp parallel for default(none) num_threads(2)  /* as req. by Ralf */ \
-          shared(aLines, n_slices, i_slice, lsfParams, slice_pixtable,         \
-                 max_iterations)
+          shared(aLines, aMaxIter, n_slices, i_slice, lsfParams, slice_pixtable)
   for (i_slice = 0; i_slice < n_slices; i_slice++) {
-    uint32_t origin
-      = (uint32_t)cpl_table_get_int(slice_pixtable[i_slice]->table,
-                                    MUSE_PIXTABLE_ORIGIN, 0, NULL);
+    cpl_errorstate es = cpl_errorstate_get();
+    uint32_t origin = cpl_table_get_int(slice_pixtable[i_slice]->table,
+                                        MUSE_PIXTABLE_ORIGIN, 0, NULL);
+    if (!cpl_errorstate_is_equal(es)) {
+      cpl_msg_error(__func__, "While processing slice/ifu index %d:", i_slice);
+      cpl_errorstate_dump(es, CPL_FALSE, NULL);
+      cpl_errorstate_set(es);
+      continue;
+    }
     unsigned short ifu = muse_pixtable_origin_get_ifu(origin);
     unsigned short slice = muse_pixtable_origin_get_slice(origin);
 
     cpl_errorstate prestate = cpl_errorstate_get();
-    lsfParams[i_slice]
-      = muse_lsf_params_fit(slice_pixtable[i_slice], aLines, max_iterations);
+    lsfParams[i_slice] = muse_lsf_params_fit(slice_pixtable[i_slice], aLines,
+                                             aMaxIter);
     if (!cpl_errorstate_is_equal(prestate)) {
       cpl_msg_error(__func__, "While processing slice %hu.%hu:", ifu, slice);
       cpl_errorstate_dump(prestate, CPL_FALSE, NULL);
@@ -217,19 +237,20 @@ muse_lsf_params_compute(cpl_table *aLines,
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    Create a header with the QC parameters for the LSF parameters.
-  @param    aMaster Computed sky parameters
-  @return   Header with the QC parameters for the sky.
+  @brief    Create a header with the QC parameters for the old LSF parameters.
+  @param    aHeader      header to output the QC
+  @param    aSlicePars   computed LSF parameters
+  @return   CPL_ERROR_NONE on success another CPL error code on failure
  */
 /*----------------------------------------------------------------------------*/
 static cpl_error_code
 muse_lsf_params_qc(cpl_propertylist *aHeader, const muse_lsf_params **aSlicePars)
 {
-  cpl_ensure_code(aHeader != NULL, CPL_ERROR_NULL_INPUT);
-  cpl_ensure_code(aSlicePars != NULL, CPL_ERROR_NULL_INPUT);
+  cpl_ensure_code(aHeader, CPL_ERROR_NULL_INPUT);
+  cpl_ensure_code(aSlicePars, CPL_ERROR_NULL_INPUT);
 
   const muse_lsf_params **det;
-  for (det = aSlicePars; *det != NULL; det++) {
+  for (det = aSlicePars; *det; det++) {
     cpl_size n_lambda = 30;
     double lambda_step = (kMuseNominalLambdaMax - kMuseNominalLambdaMin)
       / (n_lambda - 1);
@@ -259,17 +280,27 @@ muse_lsf_params_qc(cpl_propertylist *aHeader, const muse_lsf_params **aSlicePars
   return CPL_ERROR_NONE;
 } /* muse_lsf_params_qc() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Save the table with the old LSF parameters.
+  @param    aHeader       header for the table
+  @param    aSlicePars    the old LSF parameter array
+  @param    aProcessing   the processing structure
+  @param    aIFU          the IFU number
+  @return   CPL_ERROR_NONE (no error checking!)
+ */
+/*----------------------------------------------------------------------------*/
 static cpl_error_code
 muse_lsf_params_save_table(cpl_propertylist *aHeader,
-                     const muse_lsf_params **aSlicePars,
-                     muse_processing *aProcessing, int aIFU)
+                           const muse_lsf_params **aSlicePars,
+                           muse_processing *aProcessing, int aIFU)
 {
   cpl_error_code r = CPL_ERROR_NONE;
-  if (aSlicePars != NULL) {
-    cpl_frame *frame
-      = muse_processing_new_frame(aProcessing, aIFU, aHeader,
-                                  "LSF_PROFILE", CPL_FRAME_TYPE_TABLE);
-    if (frame != NULL) {
+  if (aSlicePars) {
+    cpl_frame *frame = muse_processing_new_frame(aProcessing, aIFU, aHeader,
+                                                 "LSF_PROFILE",
+                                                 CPL_FRAME_TYPE_TABLE);
+    if (frame) {
       const char *filename = cpl_frame_get_filename(frame);
       cpl_msg_info(__func__, "Saving LSF table as %s", filename);
       char *channel = cpl_sprintf("CHAN%02d", aIFU);
@@ -278,7 +309,7 @@ muse_lsf_params_save_table(cpl_propertylist *aHeader,
       r = cpl_propertylist_save(aHeader, filename, CPL_IO_CREATE);
       r = muse_lsf_params_save(aSlicePars, filename);
       if (r == CPL_ERROR_NONE) {
-#pragma omp critical(muse_processing_output_frames)
+        #pragma omp critical(muse_processing_output_frames)
         cpl_frameset_insert(aProcessing->outframes, frame);
       } else {
         cpl_frame_delete(frame);
@@ -287,7 +318,7 @@ muse_lsf_params_save_table(cpl_propertylist *aHeader,
   }
   return r;
 } /* muse_lsf_params_save_table() */
-#endif // defined(USE_LSF_PARAMS)
+#endif /* defined(USE_LSF_PARAMS) */
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -429,9 +460,9 @@ muse_lsf_compute(muse_processing *aProcessing, muse_lsf_params_t *aParams)
     cpl_table_delete(arclines);
     muse_image_delete(masterimage);
   }
-#endif // defined(USE_LSF_PARAMS)
+#endif /* defined(USE_LSF_PARAMS) */
   else {
     cpl_msg_error(__func__, "Unknown LSF method %s", aParams->method_s);
   }
   return (cpl_errorstate_is_equal(prestate))?0:-1;
-}
+} /* muse_lsf_compute() */
