@@ -89,7 +89,17 @@ muse_scipost_compute(muse_processing *aProcessing,
   prop->autocalib = MUSE_POSTPROC_AUTOCALIB_NONE;
   if (aParams->autocalib == MUSE_SCIPOST_PARAM_AUTOCALIB_DEEPFIELD) {
     prop->autocalib = MUSE_POSTPROC_AUTOCALIB_DEEPFIELD;
+  } else if (aParams->autocalib == MUSE_SCIPOST_PARAM_AUTOCALIB_USER) {
+    prop->autocalib = MUSE_POSTPROC_AUTOCALIB_USER;
+    prop->autocal_table = muse_processing_load_table(aProcessing,
+                                                     MUSE_TAG_ACAL_FACTORS, 0);
+    if (!prop->autocal_table) {
+      /* autocalib=user but no table given */
+      cpl_msg_warning(__func__, "User-provided %s could not be loaded, will "
+                      "create it from the data!", MUSE_TAG_ACAL_FACTORS);
+    }
   }
+
   /* Raman emission handling */
   prop->raman_width = aParams->raman_width;
   prop->raman_lines = muse_raman_lines_load(aProcessing);
@@ -184,6 +194,9 @@ muse_scipost_compute(muse_processing *aProcessing,
   } else if (aParams->skymethod == MUSE_SCIPOST_PARAM_SKYMETHOD_SIMPLE) {
     /* a mask could help for the simple sky subtraction */
     prop->sky_mask = muse_processing_load_mask(aProcessing, MUSE_TAG_SKY_MASK);
+  } else if (aParams->autocalib == MUSE_SCIPOST_PARAM_AUTOCALIB_DEEPFIELD) {
+    /* the sky mask is still used in case of autocalibration */
+    prop->sky_mask = muse_processing_load_mask(aProcessing, MUSE_TAG_SKY_MASK);
   }
 
   /* warn about bad combinations of inputs regarding Raman correction */
