@@ -273,6 +273,16 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
     double flux = cpl_table_get_column_mean(pts[nifu - 1]->table, MUSE_PIXTABLE_DATA)
                 * muse_pixtable_get_nrow(pts[nifu - 1]),
            scale = 1.;
+    if (nifu == 24 &&
+        muse_pfits_get_mode(pts[nifu - 1]->header) == MUSE_MODE_NFM_AO_N) {
+      /* CHAN24 in NFM is vignetted, special handling */
+      kw = cpl_sprintf(MUSE_HDR_FLAT_FLUX_SKY"%hhu", 23);
+      double flux23 = cpl_propertylist_get_double(pt->header, kw);
+      cpl_free(kw);
+      cpl_msg_debug(__func__, "NFM: using flux (%.3g) from CHAN23 for "
+                    "normalization (instead of %.3g)", flux23, flux);
+      flux = flux23;
+    } /* if IFU 24 and NFM */
     if (scales) {
       scale = atof(cpl_array_get_string(scales, nifu - 1));
       /* also recomputed the flux that is then saved in the header */
@@ -374,26 +384,6 @@ muse_twilight_image_normalize_and_fit(muse_image *aImage,
     cpl_msg_warning(__func__, "filtering cube plane failed: %s",
                     cpl_error_get_message());
   }
-#if 0
-  /* now also use a wide (~10 pixel FWHM) 2D Gaussian to smooth the output even more */
-  cpl_image *filtered2 = cpl_image_duplicate(filtered);
-  cpl_matrix *gauss = muse_matrix_new_gaussian_2d(7, 7, 10. * CPL_MATH_SIG_FWHM);
-  cpl_image_filter(filtered2, filtered, gauss, CPL_FILTER_LINEAR,
-                   CPL_BORDER_FILTER);
-  cpl_matrix_delete(gauss);
-  if (!cpl_errorstate_is_equal(prestate)) {
-    cpl_msg_warning(__func__, "filtering 2 failed: %s", cpl_error_get_message());
-  } else {
-    cpl_image_reject_from_mask(filtered2, bpm2);
-    cpl_image_fill_rejected(filtered2, 1.);
-    muse_processing_save_cimage(aProcessing, -1, filtered2, wmuse->header,
-                                "TWILIGHT_IMAGE2_SMOOTHED" /* XXX */);
-    cpl_detector_interpolate_rejected(filtered2);
-    muse_processing_save_cimage(aProcessing, -1, filtered2, wmuse->header,
-                                "TWILIGHT_IMAGE2_INTERPOLATED" /* XXX */);
-  }
-  cpl_image_delete(filtered2);
-#endif
 
   /* compute the mean of the good pixels and use that to normalize the image */
   double mean = cpl_image_get_mean(filtered);
@@ -531,28 +521,43 @@ muse_twilight_compute(muse_processing *aProcessing,
   cpl_mask_delete(bpm);
   bpm = bpm2; /* easier handle to remember */
 
-  /* apply a mask, if one was given, on top of the one that the image already has */
-  muse_mask *maskimage = muse_processing_load_mask(aProcessing,
-                                                   MUSE_TAG_VIGN_MASK);
-  /* adapt the mask to the size of the actual cube */
-  if (maskimage) {
-    cpl_mask *mask = muse_cplmask_adapt_to_image(maskimage->mask,
-                                                 cpl_imagelist_get(cube->data, 0));
-    cpl_mask_delete(maskimage->mask);
-    maskimage->mask = mask;
+  /* boolean value for the principle instrument mode, since that   *
+   * determines how we treat the vignetting (positive or negative) */
+  cpl_boolean isWFM = muse_pfits_get_mode(cube->header) != MUSE_MODE_NFM_AO_N;
+
+  /* get or create a vignetting mask */
+  muse_mask *maskimage = NULL;
+  if (isWFM) {
+    cpl_msg_debug(__func__, "WFM: use given %s, if any...", MUSE_TAG_VIGN_MASK);
+    /* apply a mask, if one was given, on top of the one that the image already has */
+    maskimage = muse_processing_load_mask(aProcessing, MUSE_TAG_VIGN_MASK);
+    if (maskimage) {
+      /* adapt the mask to the size of the actual cube */
+      cpl_mask *mask = muse_cplmask_adapt_to_image(maskimage->mask,
+                                                   cpl_imagelist_get(cube->data, 0));
+      cpl_mask_delete(maskimage->mask);
+      maskimage->mask = mask;
+    }
+  } else {
+    /* for NFM, create an internal mask that fills the whole width of *
+     * the cube and the top nmasktop pixels (default: 22 ~ 1.8 IFUs)  */
+    cpl_msg_info(__func__, "NFM: creating internal %s...", MUSE_TAG_VIGN_MASK);
+    maskimage = muse_mask_new();
+    int nx = cpl_image_get_size_x(white),
+        ny = cpl_image_get_size_y(white),
+        nmasktop = aParams->vignnfmmask > 0 && aParams->vignnfmmask < ny
+                 ? aParams->vignnfmmask : 22;
+    maskimage->header = cpl_propertylist_duplicate(cube->header);
+    maskimage->mask = cpl_mask_new(nx, ny);
+    /* mark the top pixels as masked */
+    int j;
+    for (j = ny - nmasktop; j <= ny; j++) {
+      int i;
+      for (i = 1; i <= nx; i++) {
+        cpl_mask_set(maskimage->mask, i, j, CPL_BINARY_1);
+      } /* for i (all mask columns) */
+    } /* for j (all top mask pixels) */
   }
-#if 0 // only for smoothing in wavelength direction (see below)
-  /* also compute the wavelength of each plane */
-  double crpix3 = muse_pfits_get_crpix(cube->header, 3),
-         crval3 = muse_pfits_get_crval(cube->header, 3),
-         cd33 = muse_pfits_get_cd(cube->header, 3, 3);
-  cpl_propertylist *pbla = cpl_propertylist_new();
-  cpl_propertylist_copy_property_regexp(pbla, cube->header, "^C", 0);
-  cpl_propertylist_dump(pbla, stdout);
-  fflush(stdout);
-  cpl_msg_debug(__func__, "%f %f %f", crpix3, crval3, cd33);
-  cpl_vector *pos = cpl_vector_new(npl);
-#endif
   /* and we can normalize, fit, and then normalize again, every plane in the cube */
   muse_image *image = muse_image_new();
   int ipl, npl = cpl_imagelist_get_size(cube->data);
@@ -565,33 +570,12 @@ muse_twilight_compute(muse_processing *aProcessing,
                                                            aParams->yorder,
                                                            bpm, vignmask);
     cpl_imagelist_set(cubefit->data, fit, ipl);
-#if 0 // only for smoothing in wavelength direction (see below)
-    /* wavelength of this plane */
-    cpl_vector_set(pos, ipl, ipl+1);//(ipl + 1. - crpix3) * cd33 + crval3);
-#endif
   } /* for ipl (all cube image planes) */
   image->data = NULL;
   image->stat = NULL;
   muse_image_delete(image);
   muse_datacube_delete(cube);
 
-#if 0 // XXX try to smooth with a fit in wavelenth direction?
-  cpl_image *error = cpl_image_new(cpl_image_get_size_x(white),
-                                   cpl_image_get_size_y(white),
-                                   CPL_TYPE_DOUBLE);
-  cpl_imagelist *cubefit2 = cpl_fit_imagelist_polynomial(pos, cubefit->data,
-                                                         0, 2, CPL_FALSE,
-                                                         CPL_TYPE_DOUBLE, error);
-  cpl_vector_dump(pos, stdout);
-  fflush(stdout);
-  cpl_vector_delete(pos);
-  muse_processing_save_cimage(aProcessing, -1, error, wmuse->header, "ERROR_IMAGE");
-  cpl_image_delete(error);
-  /* evaluate the fits and construct a new cube out of them */
-  cpl_imagelist_save(cubefit2, "cubefit2.fits", CPL_TYPE_UNSPECIFIED, NULL, CPL_IO_CREATE);
-  cpl_imagelist_delete(cubefit2);
-#endif
-
 #if 0 /* extra debug output for the globally fitted cube, without vignetted area */
   rc = muse_processing_save_cube(aProcessing, -1, cubefit,
                                  "TWILIGHT_CUBE_FIT", MUSE_CUBE_TYPE_FITS);
@@ -644,7 +628,7 @@ muse_twilight_compute(muse_processing *aProcessing,
     cpl_image_reject_from_mask(wdiv2, vignmask);
 
     /* search for pixels strongly deviating from unity and reject them */
-    if (aParams->vignmaskedges > 0.) {
+    if (isWFM && aParams->vignmaskedges > 0.) {
       cpl_msg_info(__func__, "Excluding strong edges (stronger than %.1f%%) "
                    "from the vignetted area", aParams->vignmaskedges * 100.);
       cpl_image *wdivX = cpl_image_duplicate(wdiv2);
@@ -718,8 +702,15 @@ muse_twilight_compute(muse_processing *aProcessing,
     /* do the smoothing in one of the several possible ways */
     cpl_image *fit = NULL;
     if (aParams->vignsmooth == MUSE_TWILIGHT_PARAM_VIGNSMOOTH_GAUSSIAN) {
+      /* possibly reset to defaults (see muse_twilight.xml) */
+      if (aParams->vignxpar < 0.) {
+        aParams->vignxpar = 10;
+      }
+      if (aParams->vignypar < 0.) {
+        aParams->vignypar = 10;
+      }
       /* use Gaussian filter to smooth the vignetted area */
-      double fwhm = (aParams->vignxpar + aParams->vignypar) / 2.;
+      double fwhm = ((double)aParams->vignxpar + (double)aParams->vignypar) / 2.;
       cpl_msg_info(__func__, "Running %.1f pix FWHM Gaussian filter...", fwhm);
       fit = cpl_image_duplicate(wdiv2);
       /* create odd halfwidth of about fwhm */
@@ -732,43 +723,25 @@ muse_twilight_compute(muse_processing *aProcessing,
       cpl_image_filter(fit, wdiv2, gauss, CPL_FILTER_LINEAR, CPL_BORDER_FILTER);
       cpl_matrix_delete(gauss);
       cpl_image_reject_from_mask(fit, cpl_image_get_bpm(wdiv2)); /* re-reject! */
-      /* again, replace values less than 1 */
-      cpl_image_threshold(fit, 1., FLT_MAX, 1., FLT_MAX);
+      if (isWFM) {
+        /* WFM had vignetting in lamp-flats, so valid values are > 1 */
+        cpl_image_threshold(fit, 1., FLT_MAX, 1., FLT_MAX);
+      } else {
+        /* NFM had vignetting in sky exposures, valid values are < 1 */
+        cpl_image_threshold(fit, -FLT_MAX, 1., -FLT_MAX, 1.);
+      }
       if (!cpl_errorstate_is_equal(prestate)) {
         cpl_msg_warning(__func__, "filtering vignetted area failed: %s",
                         cpl_error_get_message());
       }
-#if 0 /* test of trying to run the filtering on a larger area which is  *
-       * set to one towards the center of the field; implementation not *
-       * finished but only tested with a hand-edited wider mask         */
-      cpl_msg_info(__func__, "Running Gaussian filter on extended region...");
-      /* use Gaussian filter to smooth the vignetted area */
-      fit = cpl_image_duplicate(wdiv2);
-      //cpl_mask_save(cpl_image_get_bpm(wdiv2), "masktest.fits", NULL, CPL_IO_CREATE);
-      /* threshold the rest of the image */
-      cpl_image *wdiv3 = cpl_image_duplicate(wdiv2);
-      cpl_image_fill_rejected(wdiv3, 1.);
-      cpl_mask *masktest = cpl_mask_load("masktest3.fits", 0, 0);
-      cpl_image_reject_from_mask(wdiv3, masktest);
-      cpl_mask_delete(masktest);
-      /* extend the vignetted area to the top and left */
-      /* find top pixel in each column */
-      /* extend it from there 5 pix to the left and 5 pix to the top */
-      cpl_matrix *gauss = muse_matrix_new_gaussian_2d(7, 7,
-                                                      10. * CPL_MATH_SIG_FWHM);
-      cpl_errorstate prestate = cpl_errorstate_get();
-      cpl_image_filter(fit, wdiv3, gauss, CPL_FILTER_LINEAR, CPL_BORDER_FILTER);
-      cpl_image_delete(wdiv3);
-      cpl_matrix_delete(gauss);
-      cpl_image_reject_from_mask(fit, cpl_image_get_bpm(wdiv2)); /* re-reject! */
-      /* again, replace values less than 1 */
-      cpl_image_threshold(fit, 1., FLT_MAX, 1., FLT_MAX);
-      if (!cpl_errorstate_is_equal(prestate)) {
-        cpl_msg_warning(__func__, "filtering vignetted area failed: %s",
-                        cpl_error_get_message());
-      }
-#endif
     } else if (aParams->vignsmooth == MUSE_TWILIGHT_PARAM_VIGNSMOOTH_MEDIAN) {
+      /* possibly reset to defaults (see muse_twilight.xml) */
+      if (aParams->vignxpar < 0.) {
+        aParams->vignxpar = 5;
+      }
+      if (aParams->vignypar < 0.) {
+        aParams->vignypar = 5;
+      }
       cpl_msg_info(__func__, "Running %dx%d median filter...",
                    aParams->vignxpar, aParams->vignypar);
       fit = cpl_image_duplicate(wdiv2);
@@ -778,29 +751,85 @@ muse_twilight_compute(muse_processing *aProcessing,
       cpl_image_filter_mask(fit, wdiv2, mask, CPL_FILTER_MEDIAN,
                             CPL_BORDER_FILTER);
       cpl_mask_delete(mask);
-      /* again, replace values less than 1 */
-      cpl_image_threshold(fit, 1., FLT_MAX, 1., FLT_MAX);
+      if (isWFM) {
+        /* again, replace values less than 1 */
+        cpl_image_threshold(fit, 1., FLT_MAX, 1., FLT_MAX);
+      } else {
+        cpl_image_threshold(fit, -FLT_MAX, 1., -FLT_MAX, 1.);
+      }
       if (!cpl_errorstate_is_equal(prestate)) {
         cpl_msg_warning(__func__, "filtering vignetted ared failed: %s",
                         cpl_error_get_message());
       }
     } else { /* MUSE_TWILIGHT_PARAM_VIGNSMOOTH_POLYFIT */
-      cpl_msg_info(__func__, "Running polyfit...");
+      if (aParams->vignxpar < 0.) {
+        aParams->vignxpar = 4;
+      }
+      if (aParams->vignypar < 0.) {
+        aParams->vignypar = 4;
+      }
+      cpl_msg_info(__func__, "Running polyfit (orders %dx%d) ...",
+                   aParams->vignxpar, aParams->vignypar);
       /* since we want to correct only vignetting, everything in this image *
-       * should be one or above, so that we should replace lower values     */
+       * should be one or above (for WFM) or one or below (for NFM), so     *
+       * that we should unlikely values                                     */
       cpl_image *wdiv3 = cpl_image_duplicate(wdiv2);
-      cpl_image_threshold(wdiv3, 1., FLT_MAX, 1., FLT_MAX);
+      if (isWFM) {
+        cpl_image_threshold(wdiv3, 1., FLT_MAX, 1., FLT_MAX);
+      } else {
+        cpl_image_threshold(wdiv3, -FLT_MAX, 1., -FLT_MAX, 1.);
+      }
 
       /* do the fit */
       fit = muse_utils_image_fit_polynomial(wdiv3, aParams->vignxpar,
                                             aParams->vignypar);
       cpl_image_delete(wdiv3);
-      /* again, replace values less than 1 */
-      cpl_image_threshold(fit, 1., FLT_MAX, 1., FLT_MAX);
+      if (isWFM) {
+        cpl_image_threshold(fit, 1., FLT_MAX, 1., FLT_MAX);
+      } else {
+        cpl_image_threshold(fit, -FLT_MAX, 1., -FLT_MAX, 1.);
+      }
     } /* else */
 
-    /* set the original rejection mask again */
-    cpl_image_reject_from_mask(fit, vignmask);
+    if (isWFM) {
+      /* set the original rejection mask again */
+      cpl_image_reject_from_mask(fit, vignmask);
+    } else {
+      /* for NFM, reject using the originally created mask, but check *
+       * row levels before doing so: there might be a row below 1 at  *
+       * the bottom of the fit region                                 */
+      cpl_mask *mask = cpl_mask_duplicate(maskimage->mask);
+      /* the mask currently marks the non-vignetted area, so invert it */
+      cpl_mask_not(mask);
+      int j, nx = cpl_mask_get_size_x(mask),
+          ny = cpl_mask_get_size_y(mask);
+      double maxval = -DBL_MAX;
+      for (j = ny; j >= 1; j--) {
+        int nmasked = cpl_mask_count_window(mask, 1, j, nx, j);
+        if (nmasked <= 0) {
+          continue;
+        } /* if */
+        double avg = cpl_image_get_mean_window(fit, 1, j, nx, j),
+               stdev = cpl_image_get_stdev_window(fit, 1, j, nx, j),
+               median = cpl_image_get_median_window(fit, 1, j, nx, j);
+        cpl_msg_debug(__func__, "row %d (%d): %.5f +/- %.5f (%.5f)", j, nmasked,
+                      avg, stdev, median);
+        if (maxval < avg) {
+          maxval = avg;
+        } /* if */
+        if (fabs(maxval - 1.) < 0.1 && avg < maxval) {
+          cpl_msg_debug(__func__, "    (below max, removing from mask)");
+          int i;
+          for (i = 1; i <= nx; i++) {
+            cpl_mask_set(mask, i, j, CPL_BINARY_0);
+          } /* for i (all mask columns) */
+        } /* if */
+      } /* for j (all mask rows) */
+      /* now we again need the non-vignetted area, so invert it */
+      cpl_mask_not(mask);
+      cpl_image_reject_from_mask(fit, mask);
+      cpl_mask_delete(mask);
+    }
     /* fill everything but the fitted area with ones, so that we  *
      * don't add rubbish when now dividing by this vignetting fix */
     cpl_image_fill_rejected(fit, 1.);
@@ -838,12 +867,10 @@ muse_twilight_compute(muse_processing *aProcessing,
     cpl_image_delete(wdiv2);
 
     /* apply the vignetting fit on top of the twilight cube */
-    cpl_msg_info(__func__, "Combining smooth field of view and vignetted corner"
+    cpl_msg_info(__func__, "Combining smooth field of view and vignetted region"
                  " in %d planes", npl);
     for (ipl = 0; ipl < npl; ipl++) {
       cpl_image *cimage = cpl_imagelist_get(cubefit->data, ipl);
-      // XXX the best here would be to restrict the mask to the illuminated
-      //     area instead of using the full image plane!
       cpl_image_accept_all(cimage);
       double mean1 = cpl_image_get_mean(cimage);
       cpl_image_multiply(cimage, fit);
