@@ -238,7 +238,7 @@ muse_test_flux_integrate_from_cube(int argc, char **argv)
        *inext = NULL;
   muse_flux_profile_type profile = MUSE_FLUX_PROFILE_MOFFAT;
   muse_flux_selection_type select = MUSE_FLUX_SELECT_NEAREST;
-  muse_flux_smooth_type smooth = MUSE_FLUX_SMOOTH_PPOLY;
+  muse_spectrum_smooth_type smooth = MUSE_SPECTRUM_SMOOTH_PPOLY;
 
   /* argument processing */
   int i;
@@ -276,11 +276,11 @@ muse_test_flux_integrate_from_cube(int argc, char **argv)
       i++;
       if (i < argc) {
         if (strncmp(argv[i], "none", 5) == 0) {
-          smooth = MUSE_FLUX_SMOOTH_NONE;
+          smooth = MUSE_SPECTRUM_SMOOTH_NONE;
         } else if (strncmp(argv[i], "median", 7) == 0) {
-          smooth = MUSE_FLUX_SMOOTH_MEDIAN;
+          smooth = MUSE_SPECTRUM_SMOOTH_MEDIAN;
         } else if (strncmp(argv[i], "ppoly", 6) == 0) {
-          smooth = MUSE_FLUX_SMOOTH_PPOLY;
+          smooth = MUSE_SPECTRUM_SMOOTH_PPOLY;
         }
       } else {
         PRINT_USAGE(4);
@@ -720,7 +720,7 @@ int main(int argc, char **argv)
 
   /* check muse_flux_get_response_table() */
   state = cpl_errorstate_get();
-  rc = muse_flux_get_response_table(fluxobj, MUSE_FLUX_SMOOTH_MEDIAN);
+  rc = muse_flux_get_response_table(fluxobj, MUSE_SPECTRUM_SMOOTH_MEDIAN);
   cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
   cpl_test_nonnull(fluxobj->response);
   cpl_test_nonnull(fluxobj->response->table);
@@ -771,7 +771,7 @@ int main(int argc, char **argv)
                                  CPL_TRUE);
 
     state = cpl_errorstate_get();
-    rc = muse_flux_get_response_table(fobj3p, MUSE_FLUX_SMOOTH_MEDIAN);
+    rc = muse_flux_get_response_table(fobj3p, MUSE_SPECTRUM_SMOOTH_MEDIAN);
     cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
     cpl_test_nonnull(fobj3p->response);
     cpl_test_nonnull(fobj3p->response->table);
@@ -794,7 +794,7 @@ int main(int argc, char **argv)
     muse_flux_object_delete(fobj3p);
 
     state = cpl_errorstate_get();
-    rc = muse_flux_get_response_table(fobj4p, MUSE_FLUX_SMOOTH_PPOLY);
+    rc = muse_flux_get_response_table(fobj4p, MUSE_SPECTRUM_SMOOTH_PPOLY);
     cpl_test(rc == CPL_ERROR_NONE && cpl_errorstate_is_equal(state));
     cpl_test_nonnull(fobj4p->response);
     cpl_test_nonnull(fobj4p->response->table);
@@ -1034,7 +1034,7 @@ int main(int argc, char **argv)
    * (but don't do the airmass correction again!)             */
   cpl_test(muse_flux_response_compute(fo, MUSE_FLUX_SELECT_BRIGHTEST, 1.0, ref,
                                       NULL, NULL) == CPL_ERROR_NONE);
-  cpl_test(muse_flux_get_response_table(fo, MUSE_FLUX_SMOOTH_PPOLY) == CPL_ERROR_NONE);
+  cpl_test(muse_flux_get_response_table(fo, MUSE_SPECTRUM_SMOOTH_PPOLY) == CPL_ERROR_NONE);
   cpl_test_nonnull(fo->response);
 #if 0
   cpl_error_code ec = cpl_plot_column(NULL, NULL, NULL, fo->response->table,
@@ -1065,7 +1065,7 @@ int main(int argc, char **argv)
   muse_table_delete(fo->response);
   fo->response = NULL;
   cpl_table_erase_window(fo->sensitivity, 50, 5);
-  cpl_test(muse_flux_get_response_table(fo, MUSE_FLUX_SMOOTH_NONE) == CPL_ERROR_NONE);
+  cpl_test(muse_flux_get_response_table(fo, MUSE_SPECTRUM_SMOOTH_NONE) == CPL_ERROR_NONE);
   cpl_test_nonnull(fo->response);
   /* results should be roughly similar */
   mean = cpl_table_get_column_mean(fo->response->table, "response");
@@ -1083,7 +1083,7 @@ int main(int argc, char **argv)
   muse_table_delete(fo->response);
   fo->response = NULL;
   cpl_table_erase_window(fo->sensitivity, 50, 5);
-  cpl_test(muse_flux_get_response_table(fo, MUSE_FLUX_SMOOTH_PPOLY) == CPL_ERROR_NONE);
+  cpl_test(muse_flux_get_response_table(fo, MUSE_SPECTRUM_SMOOTH_PPOLY) == CPL_ERROR_NONE);
   cpl_test_nonnull(fo->response);
   /* results should be very similar to the first case */
   mean = cpl_table_get_column_mean(fo->response->table, "response");
@@ -1146,14 +1146,25 @@ int main(int argc, char **argv)
   cpl_table_set_column_unit(ptdupe->table, MUSE_PIXTABLE_DATA, "adu");
   ps = cpl_errorstate_get();
   rc = muse_flux_calibrate(ptdupe, fluxobj->response, extinction, fluxobj->telluric);
-  cpl_test(rc == CPL_ERROR_INCOMPATIBLE_INPUT && !cpl_errorstate_is_equal(ps));
+  cpl_test(rc == CPL_ERROR_BAD_FILE_FORMAT && !cpl_errorstate_is_equal(ps));
   cpl_errorstate_set(ps);
+  cpl_table_set_column_unit(ptdupe->table, MUSE_PIXTABLE_DATA, "count");
   /* the flux calibrated one should fail, too */
   rc = muse_flux_calibrate(pt, fluxobj->response, extinction, fluxobj->telluric);
-  cpl_test(rc == CPL_ERROR_INCOMPATIBLE_INPUT && !cpl_errorstate_is_equal(ps));
+  cpl_test(rc == CPL_ERROR_CONTINUE && !cpl_errorstate_is_equal(ps));
   cpl_errorstate_set(ps);
+  /* different INS.MODE */
+  char *insmode = cpl_strdup(muse_pfits_get_insmode(ptdupe->header));
+  cpl_propertylist_update_string(ptdupe->header, "ESO INS MODE", "NFM-AO-N");
+  cpl_propertylist_update_string(fluxobj->response->header, "ESO INS MODE", "WFM-AO-N");
+  ps = cpl_errorstate_get();
+  rc = muse_flux_calibrate(ptdupe, fluxobj->response, extinction, fluxobj->telluric);
+  cpl_test(rc == CPL_ERROR_INCOMPATIBLE_INPUT);
+  cpl_errorstate_set(ps);
+  cpl_propertylist_update_string(ptdupe->header, "ESO INS MODE", insmode); /* reset */
+  cpl_propertylist_update_string(fluxobj->response->header, "ESO INS MODE", insmode);
+  cpl_free(insmode);
   /* bad EXPTIME */
-  cpl_table_set_column_unit(ptdupe->table, MUSE_PIXTABLE_DATA, "count");
   cpl_propertylist_update_double(ptdupe->header, "EXPTIME", 0.); /* bias-like */
   rc = muse_flux_calibrate(ptdupe, fluxobj->response, extinction, fluxobj->telluric);
   cpl_test(rc == CPL_ERROR_ILLEGAL_INPUT);
