@@ -19,6 +19,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#define _DEFAULT_SOURCE
 #define _BSD_SOURCE /* get setenv() from stdlib.h */
 #include <stdlib.h> /* setenv() */
 #include <string.h>
@@ -236,6 +237,7 @@ int
 muse_test_wcs_calibrate_from_cube(int argc, char **argv)
 {
   cpl_errorstate state = cpl_errorstate_get();
+  cpl_msg_set_level(CPL_MSG_DEBUG);
 
   /* defaults */
   char *incube = NULL,
@@ -384,6 +386,22 @@ muse_test_wcs_calibrate_from_cube(int argc, char **argv)
       muse_datacube_delete(cube);
       PRINT_USAGE(12);
     } /* if recreating detimage failed */
+
+    /* append it to cube object as well, it needs to end up as 2nd  *
+     * reconstructed image for muse_wcs_optimize_solution() to work */
+    if (!cube->recnames) {
+      cube->recnames = cpl_array_new(1, CPL_TYPE_STRING);
+      cpl_array_set_string(cube->recnames, 0, "dummy");
+    }
+    if (!cube->recimages) {
+      cube->recimages = muse_imagelist_new();
+      muse_imagelist_set(cube->recimages, muse_image_duplicate(detimage), 0);
+    }
+    cpl_array_set_size(cube->recnames, cpl_array_get_size(cube->recnames) + 1);
+    cpl_array_set_string(cube->recnames, cpl_array_get_size(cube->recnames) - 1,
+                         MUSE_WCS_DETIMAGE_EXTNAME);
+    muse_imagelist_set(cube->recimages, detimage,
+                       muse_imagelist_get_size(cube->recimages));
   } else {
     extname = muse_pfits_get_extname(detimage->header);
   } /* else detimage from cube */
@@ -406,22 +424,19 @@ muse_test_wcs_calibrate_from_cube(int argc, char **argv)
   if (!reftable) {
     muse_datacube_delete(cube);
     muse_wcs_object_delete(wcs);
-    if (!extname) {
-      muse_image_delete(detimage);
-    }
     PRINT_USAGE(11);
   }
   /* use defaults from the muse_astrometry recipe */
+  wcs->cube = cube;
   if (detsigma < 0.) {
     cpl_msg_debug(__func__, "optimize %.3f", detsigma);
-    wcs->cube = cube;
     muse_wcs_optimize_solution(wcs, detsigma, centroid, reftable, radius,
                                faccuracy, niter, rejsigma);
-    wcs->cube = NULL; /* delete it separately */
   } else {
     cpl_msg_debug(__func__, "solve %.3f", detsigma);
     muse_wcs_solve(wcs, reftable, radius, faccuracy, niter, rejsigma);
   }
+  wcs->cube = NULL; /* delete it separately */
 
   /* prepare input filename for creation of output names */
   char *dot = strrchr(incube, '.');
@@ -437,7 +452,6 @@ muse_test_wcs_calibrate_from_cube(int argc, char **argv)
                     "%s", fn, cpl_error_get_message());
     }
     cpl_free(fn);
-    muse_image_delete(detimage);
   } /* if no extname */
   char *object = cpl_sprintf("Astrometric calibration (%s)",
                              cpl_propertylist_get_string(cube->header, "OBJECT"));
